# Generated by Django 5.1.2 on 2024-11-29 15:45

from django.db import migrations, models

import umap.models


class Migration(migrations.Migration):
    dependencies = [
        ("umap", "0024_alter_map_share_status"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datalayer",
            name="geojson",
            field=models.FileField(
                blank=True,
                null=True,
                storage=umap.models.set_storage,
                upload_to=umap.models.upload_to,
            ),
        ),
    ]



# Generated by Django 5.1.2 on 2024-12-10 16:48

from django.db import migrations, models

import umap.models


class Migration(migrations.Migration):
    dependencies = [
        ("umap", "0023_alter_datalayer_uuid"),
    ]

    operations = [
        migrations.AlterField(
            model_name="map",
            name="share_status",
            field=models.SmallIntegerField(
                choices=[
                    (0, "Draft (private)"),
                    (1, "Everyone (public)"),
                    (2, "Anyone with link"),
                    (3, "Editors and team only"),
                    (9, "Blocked"),
                    (99, "Deleted"),
                ],
                default=umap.models.get_default_share_status,
                verbose_name="share status",
            ),
        ),
    ]



# Generated by Django 5.1 on 2024-08-15 11:33

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import umap.models


class Migration(migrations.Migration):
    dependencies = [("umap", "0021_remove_map_description")]

    operations = [
        migrations.CreateModel(
            name="Team",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(max_length=200, unique=True, verbose_name="name"),
                ),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="description"),
                ),
                (
                    "users",
                    models.ManyToManyField(
                        related_name="teams", to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="map",
            name="team",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="umap.team",
                verbose_name="team",
            ),
        ),
        migrations.AlterField(
            model_name="datalayer",
            name="edit_status",
            field=models.SmallIntegerField(
                choices=[
                    (0, "Inherit"),
                    (1, "Everyone"),
                    (2, "Editors and team only"),
                    (3, "Owner only"),
                ],
                default=0,
                verbose_name="edit status",
            ),
        ),
        migrations.AlterField(
            model_name="map",
            name="edit_status",
            field=models.SmallIntegerField(
                choices=[
                    (1, "Everyone"),
                    (2, "Editors and team only"),
                    (3, "Owner only"),
                ],
                default=umap.models.get_default_edit_status,
                verbose_name="edit status",
            ),
        ),
        migrations.AlterField(
            model_name="map",
            name="share_status",
            field=models.SmallIntegerField(
                choices=[
                    (1, "Everyone (public)"),
                    (2, "Anyone with link"),
                    (3, "Editors and team only"),
                    (9, "Blocked"),
                ],
                default=umap.models.get_default_share_status,
                verbose_name="share status",
            ),
        ),
    ]



from datetime import datetime, timedelta
from pathlib import Path
from unittest import mock

import pytest
from django.core.files.base import ContentFile
from django.core.management import call_command

from .base import DataLayerFactory, MapFactory

pytestmark = pytest.mark.django_db


def test_purge_old_versions(map):
    map2 = MapFactory()
    other_layer = DataLayerFactory(map=map2)
    recent_layer = DataLayerFactory(map=map)
    old_layer = DataLayerFactory(map=map)
    older_layer = DataLayerFactory(map=map)
    other_layer.geojson.storage.save(
        Path(other_layer.geojson.storage._base_path(other_layer))
        / f"{other_layer.uuid}_1440918537.geojson",
        ContentFile("{}"),
    )
    with mock.patch("django.utils.timezone.now") as mocked:
        mocked.return_value = datetime.utcnow() - timedelta(days=7)
        old_layer.save()
        old_layer.geojson.storage.save(
            Path(old_layer.geojson.storage._base_path(old_layer))
            / f"{old_layer.uuid}_1440918537.geojson",
            ContentFile("{}"),
        )
        old_layer.geojson.storage.save(
            Path(old_layer.geojson.storage._base_path(old_layer))
            / f"{old_layer.uuid}_1440918537.geojson.gz",
            ContentFile("{}"),
        )
        mocked.return_value = datetime.utcnow() - timedelta(days=12)
        older_layer.save()
        older_layer.geojson.storage.save(
            Path(older_layer.geojson.storage._base_path(older_layer))
            / f"{older_layer.uuid}_1340918536.geojson",
            ContentFile("{}"),
        )
        older_layer.geojson.storage.save(
            Path(older_layer.geojson.storage._base_path(older_layer))
            / f"{older_layer.uuid}_1340918536.geojson.gz",
            ContentFile("{}"),
        )
    assert len(recent_layer.versions) == 1
    assert len(old_layer.versions) == 2
    assert len(older_layer.versions) == 2
    assert len(other_layer.versions) == 2
    root = old_layer.geojson.storage._base_path(old_layer)
    # Files including gz for map 1
    # recent layer geojson
    # old layer 2 geojson + 1 gzip
    # older layer 2 geojson + 1 gzip
    assert len(old_layer.geojson.storage.listdir(root)[1]) == 7
    call_command("purge_old_versions", "--days-ago=7", "--dry-run")
    assert len(recent_layer.versions) == 1
    assert len(old_layer.versions) == 2
    assert len(older_layer.versions) == 2
    assert len(other_layer.versions) == 2
    assert len(old_layer.geojson.storage.listdir(root)[1]) == 7
    call_command("purge_old_versions", "--days-ago=9")
    assert len(recent_layer.versions) == 1
    assert len(old_layer.versions) == 2
    assert len(older_layer.versions) == 2
    assert len(other_layer.versions) == 2
    assert len(old_layer.geojson.storage.listdir(root)[1]) == 7
    call_command("purge_old_versions", "--days-ago=7")
    assert len(recent_layer.versions) == 1
    assert len(old_layer.versions) == 1
    assert len(older_layer.versions) == 2
    assert len(other_layer.versions) == 2
    # Files including gz for map 1
    # recent layer geojson
    # old layer 1 geojson
    # older layer 2 geojson + 1 gz
    assert len(old_layer.geojson.storage.listdir(root)[1]) == 5
    call_command("purge_old_versions", "--days-ago=7", "--days-to-select=0")
    assert len(recent_layer.versions) == 1
    assert len(old_layer.versions) == 1
    assert len(older_layer.versions) == 1
    assert len(other_layer.versions) == 2
    # Files including gz for map 1
    # recent layer geojson
    # old layer 1 geojson
    # older layer 1 geojson
    assert len(old_layer.geojson.storage.listdir(root)[1]) == 3



# Generated by Django 5.1.7 on 2025-04-17 09:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("umap", "0027_map_tags"),
    ]

    operations = [
        migrations.AddField(
            model_name="map",
            name="is_template",
            field=models.BooleanField(
                default=False,
                help_text="This map is a template map.",
                verbose_name="save as template",
            ),
        ),
    ]



import pytest
from django.contrib.auth.models import AnonymousUser
from django.urls import reverse

from umap.forms import DEFAULT_CENTER
from umap.models import Map

from .base import MapFactory

pytestmark = pytest.mark.django_db


def test_anonymous_can_edit_if_status_anonymous(map, fake_request):
    map.edit_status = map.ANONYMOUS
    map.save()
    fake_request.user = AnonymousUser()
    assert map.can_edit(fake_request)


def test_anonymous_cannot_edit_if_not_status_anonymous(map, fake_request):
    map.edit_status = map.OWNER
    map.save()
    fake_request.user = AnonymousUser()
    assert not map.can_edit(fake_request)


def test_non_editors_can_edit_if_status_anonymous(map, user, fake_request):
    assert map.owner != user
    map.edit_status = map.ANONYMOUS
    map.save()
    fake_request.user = user
    assert map.can_edit(fake_request)


def test_non_editors_cannot_edit_if_not_status_anonymous(map, user, fake_request):
    map.edit_status = map.OWNER
    map.save()
    fake_request.user = user
    assert not map.can_edit(fake_request)


def test_editors_cannot_edit_if_status_owner(map, user, fake_request):
    map.edit_status = map.OWNER
    map.editors.add(user)
    map.save()
    fake_request.user = user
    assert not map.can_edit(fake_request)


def test_editors_can_edit_if_status_collaborators(map, user, fake_request):
    map.edit_status = map.COLLABORATORS
    map.editors.add(user)
    map.save()
    fake_request.user = user
    assert map.can_edit(fake_request)


def test_team_members_cannot_edit_if_status_owner(map, user, team, fake_request):
    user.teams.add(team)
    user.save()
    map.edit_status = map.OWNER
    map.team = team
    map.save()
    fake_request.user = user
    assert not map.can_edit(fake_request)


def test_team_members_can_edit_if_status_collaborators(map, user, team, fake_request):
    user.teams.add(team)
    user.save()
    map.edit_status = map.COLLABORATORS
    map.team = team
    map.save()
    fake_request.user = user
    assert map.can_edit(fake_request)


def test_logged_in_user_should_be_allowed_for_anonymous_map_with_anonymous_edit_status(
    map, user, rf
):  # noqa
    map.owner = None
    map.edit_status = map.ANONYMOUS
    map.save()
    url = reverse("map_update", kwargs={"map_id": map.pk})
    request = rf.get(url)
    request.user = user
    assert map.can_edit(request)


def test_anonymous_user_should_not_be_allowed_for_anonymous_map(
    map, user, fake_request
):
    map.owner = None
    map.edit_status = map.OWNER
    map.save()
    fake_request.user = AnonymousUser()
    assert not map.can_edit(fake_request)


def test_clone_should_return_new_instance(map, user):
    clone = map.clone()
    assert map.pk != clone.pk
    assert "Clone of " + map.name == clone.name
    assert map.settings == clone.settings
    assert map.center == clone.center
    assert map.zoom == clone.zoom
    assert map.licence == clone.licence


def test_clone_should_keep_editors(map, user):
    map.editors.add(user)
    clone = map.clone()
    assert map.pk != clone.pk
    assert user in map.editors.all()
    assert user in clone.editors.all()


def test_clone_should_keep_team(map, user, team):
    map.team = team
    map.save()
    clone = map.clone()
    assert map.pk != clone.pk
    assert clone.team == team


def test_clone_should_update_owner_if_passed(map, user):
    clone = map.clone(owner=user)
    assert map.pk != clone.pk
    assert map.owner != clone.owner
    assert user == clone.owner


def test_clone_should_clone_datalayers_and_features_too(map, user, datalayer):
    clone = map.clone()
    assert map.pk != clone.pk
    assert map.datalayer_set.count() == 1
    assert clone.datalayer_set.count() == 1
    other = clone.datalayer_set.all()[0]
    assert datalayer in map.datalayer_set.all()
    assert other.pk != datalayer.pk
    assert other.name == datalayer.name
    assert other.geojson is not None
    assert other.geojson.path != datalayer.geojson.path


def test_publicmanager_should_get_only_public_maps(map, user, licence):
    map.share_status = map.PUBLIC
    open_map = MapFactory(owner=user, licence=licence, share_status=Map.OPEN)
    private_map = MapFactory(owner=user, licence=licence, share_status=Map.PRIVATE)
    assert map in Map.public.all()
    assert open_map not in Map.public.all()
    assert private_map not in Map.public.all()


def test_can_change_default_edit_status(user, settings):
    map = MapFactory(owner=user)
    assert map.edit_status == Map.OWNER
    settings.UMAP_DEFAULT_EDIT_STATUS = Map.COLLABORATORS
    map = MapFactory(owner=user)
    assert map.edit_status == Map.COLLABORATORS


def test_can_change_default_share_status(user, settings):
    map = Map.objects.create(owner=user, center=DEFAULT_CENTER)
    assert map.share_status == Map.DRAFT
    settings.UMAP_DEFAULT_SHARE_STATUS = Map.PUBLIC
    map = Map.objects.create(owner=user, center=DEFAULT_CENTER)
    map = MapFactory(owner=user)
    assert map.share_status == Map.PUBLIC


def test_move_to_trash(user, map):
    map.move_to_trash()
    map.save()
    reloaded = Map.objects.get(pk=map.pk)
    assert reloaded.share_status == Map.DELETED



# Generated by Django 5.0.2 on 2024-02-13 14:04

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("umap", "0016_pictogram_category"),
    ]

    operations = [
        migrations.RunSQL(
            "UPDATE social_auth_usersocialauth "
            "SET provider = 'openstreetmap-oauth2' WHERE provider = 'openstreetmap'"
        ),
    ]



from playwright.sync_api import expect

from umap.models import Map

from ..base import DataLayerFactory


def test_dynamic_remote_data(page, live_server, tilelayer, map):
    data = [
        {
            "type": "FeatureCollection",
            "features": [
                {
                    "type": "Feature",
                    "properties": {"name": "Point 2"},
                    "geometry": {
                        "type": "Point",
                        "coordinates": [4.3375, 11.2707],
                    },
                }
            ],
        },
        {
            "type": "FeatureCollection",
            "features": [
                {
                    "type": "Feature",
                    "properties": {"name": "Point 1"},
                    "geometry": {
                        "type": "Point",
                        "coordinates": [4.3375, 12.2707],
                    },
                }
            ],
        },
    ]

    def handle(route):
        route.fulfill(json=data.pop())

    settings = {
        "remoteData": {
            "url": "https://remote.org/data.json",
            "format": "geojson",
            "dynamic": True,
        },
        "showLabel": True,
    }
    DataLayerFactory(map=map, settings=settings)
    map.edit_status = Map.ANONYMOUS
    map.settings["properties"]["zoom"] = 6
    map.settings["geometry"] = {
        "type": "Point",
        "coordinates": [5, 12],
    }
    map.save()

    # Intercept the route to the proxy
    page.route("https://remote.org/data.json", handle)
    page.goto(f"{live_server.url}{map.get_absolute_url()}")

    expect(page.get_by_role("tooltip", name="Point 1")).to_be_visible()

    # Now drag the map
    map_el = page.locator("#map")
    map_el.drag_to(
        map_el,
        source_position={"x": 100, "y": 100},
        target_position={"x": 110, "y": 110},
    )

    expect(page.get_by_role("tooltip", name="Point 2")).to_be_visible()
    # Needed otherwise it found two (!) tooltip with name "Point 1"…
    page.wait_for_timeout(300)
    expect(page.get_by_role("tooltip", name="Point 1")).to_be_hidden()

    # Map must not be dirty
    page.get_by_role("button", name="Edit").click()
    expect(page.locator(".edit-undo")).to_be_disabled()



from copy import deepcopy

import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


def getColors(elements):
    return [
        el.evaluate("e => window.getComputedStyle(e).backgroundColor")
        for el in elements.all()
    ]


DATALAYER_DATA1 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "mytype": "even",
                "name": "Point 2",
                "mynumber": 10,
                "myboolean": True,
                "mydate": "2024/04/14 12:19:17",
                "maybeempty": "not empty",
            },
            "geometry": {"type": "Point", "coordinates": [0.065918, 48.385442]},
        },
        {
            "type": "Feature",
            "properties": {
                "mytype": "odd",
                "name": "Point 1",
                "mynumber": 12,
                "myboolean": False,
                "mydate": "2024/03/13 12:20:20",
                "maybeempty": "",
            },
            "geometry": {"type": "Point", "coordinates": [3.55957, 49.767074]},
        },
    ],
    "_umap_options": {
        "name": "Calque 1",
    },
}


DATALAYER_DATA2 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "mytype": "even",
                "name": "Point 4",
                "mynumber": 10,
                "myboolean": "true",
                "mydate": "2024/08/18 13:14:15",
                "maybeempty": None,
            },
            "geometry": {"type": "Point", "coordinates": [0.856934, 45.290347]},
        },
        {
            "type": "Feature",
            "properties": {
                "mytype": "odd",
                "name": "Point 3",
                "mynumber": 14,
                "mydate": "2024-04-14T10:19:17.000Z",
            },
            "geometry": {"type": "Point", "coordinates": [4.372559, 47.945786]},
        },
        {
            "type": "Feature",
            "properties": {
                "mytype": "odd",
                "name": "Point 5",
                "mynumber": 10,
                "mydate": "2024-04-14T10:19:17.000Z",
                "myboolean": "notaboolean",
                "maybeempty": "foo",
            },
            "geometry": {"type": "Point", "coordinates": [4.1, 47.3]},
        },
    ],
    "_umap_options": {
        "name": "Calque 2",
    },
}


def test_simple_equal_rule_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "mytype=odd", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 3


def test_simple_not_equal_rule_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "mytype!=even", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 3


def test_gt_rule_with_number_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "mynumber>10", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 2


def test_lt_rule_with_number_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "mynumber<14", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 4


def test_lt_rule_with_float_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "mynumber<12.3", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 4


def test_equal_rule_with_boolean_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "myboolean=true", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 2


def test_equal_rule_with_boolean_not_true_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "myboolean!=true", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 3


def test_equal_rule_with_boolean_false_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "myboolean=false", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 1


def test_equal_rule_with_boolean_not_false_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "myboolean!=false", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 4


def test_empty_rule_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "maybeempty=", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 3


def test_not_empty_rule_at_load(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "maybeempty!=", "options": {"color": "aliceblue"}}
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 2


def test_can_create_new_rule(live_server, page, openmap):
    DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    DataLayerFactory(map=openmap, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    page.get_by_role("button", name="Edit").click()
    page.get_by_role("button", name="Map advanced properties").click()
    page.get_by_text("Conditional style rules").click()
    page.get_by_role("button", name="Add rule").click()
    page.locator("input[name=condition]").click()
    page.locator("input[name=condition]").fill("mytype=odd")
    page.locator(".umap-field-color .define").first.click()
    page.get_by_title("AliceBlue").first.click()
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 3
    page.locator(".edit-undo").click()
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 0


def test_can_deactive_rule_from_list(live_server, page, openmap):
    openmap.settings["properties"]["rules"] = [
        {"condition": "mytype=odd", "options": {"color": "aliceblue"}}
    ]
    openmap.save()
    DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    DataLayerFactory(map=openmap, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 3
    page.get_by_role("button", name="Edit").click()
    page.get_by_role("button", name="Map advanced properties").click()
    page.get_by_text("Conditional style rules").click()
    page.get_by_role("button", name="Toggle rule").click()
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 0
    page.get_by_role("button", name="Toggle rule").click()
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 3


def test_autocomplete_datalist(live_server, page, openmap):
    DataLayerFactory(map=openmap, data=DATALAYER_DATA1)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit#6/48.948/1.670")
    page.get_by_role("button", name="Map advanced properties").click()
    page.locator("summary").filter(has_text="Conditional style rules").click()
    page.get_by_role("button", name="Add rule").click()
    panel = page.locator(".panel.right.on")
    datalist = panel.locator(".umap-field-condition datalist option")
    expect(datalist).to_have_count(6)
    values = {option.inner_text() for option in datalist.all()}
    assert values == {"myboolean", "mytype", "mynumber", "mydate", "name", "maybeempty"}
    page.get_by_placeholder("key=value or key!=value").fill("mytype")
    expect(datalist).to_have_count(4)
    values = {option.inner_text() for option in datalist.all()}
    assert values == {"mytype=", "mytype!=", "mytype>", "mytype<"}
    page.get_by_placeholder("key=value or key!=value").fill("mytype=")
    expect(datalist).to_have_count(2)
    values = {option.inner_text() for option in datalist.all()}
    assert values == {"mytype=even", "mytype=odd"}


def test_can_combine_rules(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "mytype=odd", "options": {"color": "aliceblue"}},
        {"condition": "mynumber>10", "options": {"iconClass": "Drop"}},
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    drops = page.locator(".umap-drop-icon .icon_container")
    expect(markers).to_have_count(5)
    expect(drops).to_have_count(2)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 3
    colors = getColors(drops)
    assert colors.count("rgb(240, 248, 255)") == 2


def test_first_matching_rule_wins_on_given_property(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "mytype=odd", "options": {"color": "aliceblue"}},
        {"condition": "mytype!=even", "options": {"color": "darkred"}},
    ]
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    assert colors.count("rgb(240, 248, 255)") == 3


def test_rules_from_datalayer(live_server, page, map):
    map.settings["properties"]["rules"] = [
        {"condition": "mytype=odd", "options": {"color": "darkred"}}
    ]
    map.save()
    data = deepcopy(DATALAYER_DATA1)
    data["_umap_options"]["rules"] = [
        {"condition": "mytype=odd", "options": {"color": "aliceblue"}}
    ]
    DataLayerFactory(map=map, data=data)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(5)
    colors = getColors(markers)
    # Alice Blue should only affect layer 1
    assert colors.count("rgb(240, 248, 255)") == 1
    # Dark Red as for map global rules
    assert colors.count("rgb(139, 0, 0)") == 2



import csv
from datetime import datetime

from django.contrib.auth.admin import UserAdmin as UserAdminBase
from django.contrib.auth.models import User
from django.contrib.gis import admin
from django.http import HttpResponse
from django.utils.translation import gettext_lazy as _

from .models import DataLayer, Licence, Map, Pictogram, Team, TileLayer


class CSVExportMixin:
    actions = ["as_csv"]

    @admin.action(description=_("CSV Export"))
    def as_csv(self, request, queryset):
        modelname = queryset.model.__name__.lower()
        filename = f"umap_{modelname}_{datetime.now().isoformat()}.csv"
        response = HttpResponse(
            content_type="text/csv",
            headers={"Content-Disposition": f'attachment; filename="{filename}"'},
        )

        def get_cell(user, field):
            if hasattr(self, field):
                return getattr(self, field)(user)
            return getattr(user, field)

        writer = csv.writer(response)
        writer.writerow(self.csv_fields)
        for user in queryset:
            writer.writerow(get_cell(user, field) for field in self.csv_fields)
        return response


class TileLayerAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "rank",
    )
    list_editable = ("rank",)


class MapAdmin(CSVExportMixin, admin.GISModelAdmin):
    search_fields = ("name",)
    autocomplete_fields = ("owner", "editors")
    list_filter = ("share_status",)
    csv_fields = (
        "pk",
        "name",
        "center",
        "zoom",
        "created_at",
        "modified_at",
        "edit_status",
        "share_status",
        "owner_id",
        "team_id",
    )


class PictogramAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "category",
    )
    list_editable = ("category",)
    list_filter = ("category",)


class TeamAdmin(CSVExportMixin, admin.ModelAdmin):
    csv_fields = [
        "pk",
        "name",
        "users_count",
    ]
    list_display = list(admin.ModelAdmin.list_display) + ["users_count"]
    filter_horizontal = ("users",)

    def users_count(self, obj):
        return obj.users.count()


class UserAdmin(CSVExportMixin, UserAdminBase):
    csv_fields = [
        "pk",
        "username",
        "email",
        "first_name",
        "last_name",
        "last_login",
        "date_joined",
        "maps_count",
        "user_teams",
    ]
    list_display = list(UserAdminBase.list_display) + ["maps_count", "user_teams"]

    def maps_count(self, obj):
        # owner maps + maps as editor
        return obj.owned_maps.count() + obj.map_set.count()

    def user_teams(self, obj):
        return " ; ".join(obj.teams.values_list("name", flat=True))


admin.site.register(Map, MapAdmin)
admin.site.register(DataLayer)
admin.site.register(Pictogram, PictogramAdmin)
admin.site.register(TileLayer, TileLayerAdmin)
admin.site.register(Licence)
admin.site.register(Team, TeamAdmin)
admin.site.unregister(User)
admin.site.register(User, UserAdmin)



import sys
from datetime import datetime, timedelta

from django.conf import settings
from django.core.management.base import BaseCommand

from umap.models import DataLayer


class Command(BaseCommand):
    help = "Delete versions from datalayers which last modified date is x days ago. Eg.: umap purge_old_versions --days-ago 30"

    def add_arguments(self, parser):
        parser.add_argument(
            "--days-ago",
            help="Select datalayers which where last modified that many days ago",
            default=360,
            type=int,
        )
        parser.add_argument(
            "--days-to-select",
            help="How many days before `days-ago` to consider",
            default=1,
            type=int,
        )
        parser.add_argument(
            "--dry-run",
            help="Pretend to delete but just report",
            action="store_true",
        )

    def handle(self, *args, **options):
        if settings.STORAGES["data"]["BACKEND"] != "umap.storage.fs.FSDataStorage":
            msg = (
                "This command is only available for filesystem storage. "
                "For S3 storage, use lifecycle rule in the bucket."
            )
            sys.exit(msg)
        end = (datetime.utcnow() - timedelta(days=options["days_ago"] - 1)).date()
        print(f"Deleting versions for datalayer unmodified since {end}")
        filters = {"modified_at__lt": end}
        if options["days_to_select"]:
            start = end - timedelta(days=options["days_to_select"])
            filters = {"modified_at__date__range": [start, end]}
        datalayers = DataLayer.objects.filter(**filters)
        print(f"Selected {len(datalayers)} datalayers")
        total_deleted = 0
        for layer in datalayers:
            layer_id = layer.uuid
            layer_name = layer.name
            last_modified = layer.modified_at.date()
            deleted = 0
            if not options["dry_run"]:
                deleted = layer.geojson.storage.purge_old_versions(layer, keep=1)
                layer.geojson.storage.purge_gzip(layer)
                total_deleted += deleted
            print(
                f"Deleted {deleted} old versions of `{layer_name}` ({layer_id}), unmodified since {last_modified}"
            )
        if not options["dry_run"]:
            print(f"Successfully deleted {total_deleted} geojson files.")
        else:
            print(f"The command would delete {total_deleted} geojson files.")



import pytest
from django.contrib.auth import get_user_model
from django.urls import reverse

from umap.models import Map

from .base import MapFactory, UserFactory

User = get_user_model()

pytestmark = pytest.mark.django_db


def test_user_dashboard_is_restricted_to_logged_in(client):
    response = client.get(reverse("user_dashboard"))
    assert response.status_code == 302
    assert response["Location"] == "/en/login/?next=/en/me"


def test_user_dashboard_display_user_maps(client, map):
    client.login(username=map.owner.username, password="123123")
    response = client.get(reverse("user_dashboard"))
    assert response.status_code == 200
    body = response.content.decode()
    assert map.name in body
    assert f"{map.get_absolute_url()}?edit" in body
    assert f"{map.get_absolute_url()}?share" in body
    assert f"/map/{map.pk}/download" in body
    assert "Everyone (public)" in body
    assert "Owner only" in body


def test_user_dashboard_do_not_display_blocked_user_maps(client, map):
    map.share_status = Map.BLOCKED
    map.save()
    client.login(username=map.owner.username, password="123123")
    response = client.get(reverse("user_dashboard"))
    assert response.status_code == 200
    body = response.content.decode()
    assert map.name not in body


def test_user_dashboard_do_not_display_deleted_user_maps(client, map):
    map.share_status = Map.DELETED
    map.save()
    client.login(username=map.owner.username, password="123123")
    response = client.get(reverse("user_dashboard"))
    assert response.status_code == 200
    body = response.content.decode()
    assert map.name not in body


@pytest.mark.parametrize("share_status", [Map.DRAFT, Map.PRIVATE, Map.PUBLIC, Map.OPEN])
def test_user_dashboard_display_user_team_maps(client, map, team, user, share_status):
    user.teams.add(team)
    user.save()
    map.team = team
    map.share_status = share_status
    map.save()
    assert map.owner != user
    client.login(username=user.username, password="123123")
    response = client.get(reverse("user_dashboard"))
    assert response.status_code == 200
    body = response.content.decode()
    assert map.name in body
    assert map.get_absolute_url() in body


def test_user_dashboard_display_user_maps_distinct(client, map):
    # cf https://github.com/umap-project/umap/issues/1325
    anonymap = MapFactory(name="Map witout owner should not appear")
    user1 = UserFactory(username="user1")
    user2 = UserFactory(username="user2")
    map.editors.add(user1)
    map.editors.add(user2)
    map.save()
    client.login(username=map.owner.username, password="123123")
    response = client.get(reverse("user_dashboard"))
    assert response.status_code == 200
    body = response.content.decode()
    assert body.count(f'<a href="/en/map/test-map_{map.pk}">test map</a>') == 1
    assert body.count(anonymap.name) == 0


def test_user_dashboard_search(client, map):
    new_map = MapFactory(name="A map about bicycle", owner=map.owner)
    client.login(username=map.owner.username, password="123123")
    response = client.get(f"{reverse('user_dashboard')}?q=bicycle")
    assert response.status_code == 200
    body = response.content.decode()
    assert map.name not in body
    assert new_map.name in body


def test_user_dashboard_search_empty(client, map):
    client.login(username=map.owner.username, password="123123")
    response = client.get(f"{reverse('user_dashboard')}?q=car")
    assert response.status_code == 200
    body = response.content.decode()
    assert map.name not in body
    assert "No map found." in body



# Generated by Django 1.10.3 on 2016-11-26 16:11
from __future__ import unicode_literals

from django.db import migrations


def add_licence(apps, *args):
    Licence = apps.get_model("umap", "Licence")
    if Licence.objects.count():
        return
    Licence(name="ODbL", details="http://opendatacommons.org/licenses/odbl/").save()


class Migration(migrations.Migration):
    dependencies = [
        ("umap", "0003_add_tilelayer"),
    ]

    operations = [
        migrations.RunPython(add_licence),
    ]



import json
import re

import pytest
from django.core.files.base import ContentFile
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


def set_options(datalayer, **options):
    # For now we need to change both the DB and the FS…
    datalayer.settings.update(options)
    data = json.load(datalayer.geojson.file)
    data["_umap_options"].update(**options)
    datalayer.geojson = ContentFile(json.dumps(data), "foo.json")
    datalayer.save()


def test_honour_displayOnLoad_false(map, live_server, datalayer, page):
    set_options(datalayer, displayOnLoad=False)
    page.goto(f"{live_server.url}{map.get_absolute_url()}?onLoadPanel=datalayers")
    expect(page.locator(".leaflet-marker-icon")).to_be_hidden()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    layers_off = page.locator(".umap-browser .datalayer.off")
    expect(layers).to_have_count(1)
    expect(layers_off).to_have_count(1)
    page.get_by_role("button", name="Open browser").click()
    page.get_by_label("Zoom in").click()
    expect(markers).to_be_hidden()
    page.get_by_title("Show/hide layer").click()
    expect(layers_off).to_have_count(0)
    expect(markers).to_be_visible()


def test_should_honour_fromZoom(live_server, map, datalayer, page):
    set_options(datalayer, displayOnLoad=True, fromZoom=6)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#5/48.55/14.68")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_be_hidden()
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.55/14.68")
    expect(page).to_have_url(re.compile(r".*#6/48\..+/14\..+"))
    expect(markers).to_be_visible()
    page.get_by_label("Zoom out").click()
    expect(markers).to_be_hidden()
    page.get_by_label("Zoom in").click()
    expect(page).to_have_url(re.compile(r".*#6/48\..+/14\..+"))
    expect(markers).to_be_visible()
    page.get_by_label("Zoom in").click()
    expect(page).to_have_url(re.compile(r".*#7/48\..+/14\..+"))
    expect(markers).to_be_visible()


def test_should_honour_toZoom(live_server, map, datalayer, page, new_page):
    set_options(datalayer, displayOnLoad=True, toZoom=6)
    # Loading at zoom 7 should not show the marker
    page.goto(f"{live_server.url}{map.get_absolute_url()}#7/48.55/14.68")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_be_hidden()

    # Loading at zoom 6 should show the marker
    page2 = new_page()
    markers = page2.locator(".leaflet-marker-icon")
    page2.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.55/14.68")
    expect(page2).to_have_url(re.compile(r".*#6/48\..+/14\..+"))
    expect(markers).to_be_visible()

    # Now try to unzoom/rezoom and check that markers show/hide accordingly.
    page2.get_by_label("Zoom out").click()
    expect(page2).to_have_url(re.compile(r".*#5/48\..+/14\..+"))
    expect(markers).to_be_visible()
    page2.get_by_label("Zoom in").click()
    expect(page2).to_have_url(re.compile(r".*#6/48\..+/14\..+"))
    expect(markers).to_be_visible()
    page2.get_by_label("Zoom in").click()
    expect(page2).to_have_url(re.compile(r".*#7/48\..+/14\..+"))
    expect(markers).to_be_hidden()


def test_should_honour_color_variable(live_server, map, page):
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"mycolor": "aliceblue", "name": "Point 4"},
                "geometry": {"type": "Point", "coordinates": [0.856934, 45.290347]},
            },
            {
                "type": "Feature",
                "properties": {"name": "a polygon", "mycolor": "tomato"},
                "geometry": {
                    "type": "Polygon",
                    "coordinates": [
                        [
                            [2.12, 49.57],
                            [1.08, 49.02],
                            [2.51, 47.55],
                            [3.19, 48.77],
                            [2.12, 49.57],
                        ]
                    ],
                },
            },
        ],
        "_umap_options": {
            "name": "Calque 2",
            "color": "{mycolor}",
            "fillColor": "{mycolor}",
        },
    }
    DataLayerFactory(map=map, data=data)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/47.5/2.5")
    expect(page.locator(".leaflet-overlay-pane path[fill='tomato']"))
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_css("background-color", "rgb(240, 248, 255)")


def test_datalayers_in_query_string(live_server, datalayer, map, page):
    map.settings["properties"]["onLoadPanel"] = "datalayers"
    map.save()
    with_old_id = DataLayerFactory(old_id=134, map=map, name="with old id")
    set_options(with_old_id, name="with old id")
    visible = page.locator(".umap-browser .datalayer:not(.off) .datalayer-name")
    hidden = page.locator(".umap-browser .datalayer.off .datalayer-name")
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(visible).to_have_count(2)
    expect(hidden).to_have_count(0)
    page.goto(f"{live_server.url}{map.get_absolute_url()}?datalayers={datalayer.pk}")
    expect(visible).to_have_count(1)
    expect(visible).to_have_text(datalayer.name)
    expect(hidden).to_have_count(1)
    expect(hidden).to_have_text(with_old_id.name)
    page.goto(
        f"{live_server.url}{map.get_absolute_url()}?datalayers={with_old_id.old_id}"
    )
    expect(visible).to_have_count(1)
    expect(visible).to_have_text(with_old_id.name)
    expect(hidden).to_have_count(1)
    expect(hidden).to_have_text(datalayer.name)



# Generated by Django 2.0.5 on 2018-05-19 09:27

import django.contrib.gis.db.models.fields
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import umap.fields
import umap.models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="DataLayer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200, verbose_name="name")),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="description"),
                ),
                (
                    "geojson",
                    models.FileField(
                        blank=True, null=True, upload_to=umap.models.upload_to
                    ),
                ),
                (
                    "display_on_load",
                    models.BooleanField(
                        default=False,
                        help_text="Display this layer on load.",
                        verbose_name="display on load",
                    ),
                ),
                ("rank", models.SmallIntegerField(default=0)),
            ],
            options={
                "ordering": ("rank",),
            },
        ),
        migrations.CreateModel(
            name="Licence",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200, verbose_name="name")),
                (
                    "details",
                    models.URLField(
                        help_text="Link to a page where the licence is detailed.",
                        verbose_name="details",
                    ),
                ),
            ],
            options={
                "ordering": ("name",),
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Map",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200, verbose_name="name")),
                ("slug", models.SlugField()),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="description"),
                ),
                (
                    "center",
                    django.contrib.gis.db.models.fields.PointField(
                        geography=True, srid=4326, verbose_name="center"
                    ),
                ),
                ("zoom", models.IntegerField(default=7, verbose_name="zoom")),
                (
                    "locate",
                    models.BooleanField(
                        default=False,
                        help_text="Locate user on load?",
                        verbose_name="locate",
                    ),
                ),
                ("modified_at", models.DateTimeField(auto_now=True)),
                (
                    "edit_status",
                    models.SmallIntegerField(
                        choices=[
                            (1, "Everyone can edit"),
                            (2, "Only editors can edit"),
                            (3, "Only owner can edit"),
                        ],
                        default=3,
                        verbose_name="edit status",
                    ),
                ),
                (
                    "share_status",
                    models.SmallIntegerField(
                        choices=[
                            (1, "everyone (public)"),
                            (2, "anyone with link"),
                            (3, "editors only"),
                        ],
                        default=1,
                        verbose_name="share status",
                    ),
                ),
                (
                    "settings",
                    umap.fields.DictField(
                        blank=True, null=True, verbose_name="settings"
                    ),
                ),
                (
                    "editors",
                    models.ManyToManyField(
                        blank=True, to=settings.AUTH_USER_MODEL, verbose_name="editors"
                    ),
                ),
                (
                    "licence",
                    models.ForeignKey(
                        default=umap.models.get_default_licence,
                        help_text="Choose the map licence.",
                        on_delete=django.db.models.deletion.SET_DEFAULT,
                        to="umap.Licence",
                        verbose_name="licence",
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="owned_maps",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="owner",
                    ),
                ),
            ],
            options={
                "ordering": ("name",),
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Pictogram",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200, verbose_name="name")),
                ("attribution", models.CharField(max_length=300)),
                ("pictogram", models.ImageField(upload_to="pictogram")),
            ],
            options={
                "ordering": ("name",),
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TileLayer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200, verbose_name="name")),
                (
                    "url_template",
                    models.CharField(
                        help_text="URL template using OSM tile format", max_length=200
                    ),
                ),
                ("minZoom", models.IntegerField(default=0)),
                ("maxZoom", models.IntegerField(default=18)),
                ("attribution", models.CharField(max_length=300)),
                (
                    "rank",
                    models.SmallIntegerField(
                        blank=True,
                        help_text="Order of the tilelayers in the edit box",
                        null=True,
                    ),
                ),
            ],
            options={
                "ordering": ("rank", "name"),
            },
        ),
        migrations.AddField(
            model_name="map",
            name="tilelayer",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="maps",
                to="umap.TileLayer",
                verbose_name="background",
            ),
        ),
        migrations.AddField(
            model_name="datalayer",
            name="map",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="umap.Map"
            ),
        ),
    ]



import platform
import re

from playwright.sync_api import expect

from umap.models import DataLayer

from ..base import DataLayerFactory


def test_should_have_fieldset_for_layer_type_properties(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")

    # Open DataLayers list
    page.get_by_title("Manage layers").click()

    # Create a layer
    page.get_by_title("Add a layer").click()
    page.locator("input[name=name]").fill("Layer 1")

    select = page.locator(".panel.on .umap-field-type select")
    expect(select).to_be_visible()

    choropleth_header = page.get_by_text("Choropleth: settings")
    heat_header = page.get_by_text("Heatmap: settings")
    cluster_header = page.get_by_text("Clustered: settings")
    expect(choropleth_header).to_be_hidden()
    expect(heat_header).to_be_hidden()
    expect(cluster_header).to_be_hidden()

    # Switching to Choropleth should add a dedicated fieldset
    select.select_option("Choropleth")
    expect(choropleth_header).to_be_visible()
    expect(heat_header).to_be_hidden()
    expect(cluster_header).to_be_hidden()

    select.select_option("Heat")
    expect(heat_header).to_be_visible()
    expect(choropleth_header).to_be_hidden()
    expect(cluster_header).to_be_hidden()

    select.select_option("Cluster")
    expect(cluster_header).to_be_visible()
    expect(choropleth_header).to_be_hidden()
    expect(heat_header).to_be_hidden()

    select.select_option("Default")
    expect(choropleth_header).to_be_hidden()
    expect(heat_header).to_be_hidden()
    expect(cluster_header).to_be_hidden()


def test_cancel_deleting_datalayer_should_restore(
    live_server, openmap, datalayer, page
):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(1)
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel.right").get_by_title("Delete layer").click()
    expect(markers).to_have_count(0)
    expect(page.get_by_text("test datalayer")).to_be_hidden()
    page.locator(".edit-undo").click()
    expect(markers).to_have_count(1)
    expect(page.locator(".umap-browser").get_by_text("test datalayer")).to_be_visible()


def test_can_clone_datalayer(live_server, openmap, login, datalayer, page):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(1)
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel.right").get_by_title("Edit", exact=True).click()
    page.get_by_text("Advanced actions").click()
    page.get_by_role("button", name="Clone").click()
    expect(layers).to_have_count(2)
    expect(markers).to_have_count(2)


def test_can_change_icon_class(live_server, openmap, page):
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"name": "Point 4"},
                "geometry": {"type": "Point", "coordinates": [0.856934, 45.290347]},
            },
        ],
    }
    DataLayerFactory(map=openmap, data=data)
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit#6/45.3/1")
    expect(page.locator(".umap-div-icon")).to_be_visible()
    page.get_by_role("button", name="Manage layers").click()
    expect(page.locator(".umap-circle-icon")).to_be_hidden()
    page.locator(".panel.right").get_by_title("Edit", exact=True).click()
    page.get_by_text("Shape properties").click()
    page.locator(".umap-field-iconClass button.define").click()
    page.get_by_text("Circle", exact=True).click()
    expect(page.locator(".umap-circle-icon")).to_be_visible()
    expect(page.locator(".umap-div-icon")).to_be_hidden()


def test_can_change_name(live_server, openmap, page, datalayer):
    page.goto(
        f"{live_server.url}{openmap.get_absolute_url()}?edit&onLoadPanel=databrowser"
    )
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel.right").get_by_title("Edit", exact=True).click()
    expect(page.locator(".umap-is-dirty")).to_be_hidden()
    page.locator('input[name="name"]').click()
    page.locator('input[name="name"]').press("Control+a")
    page.locator('input[name="name"]').fill("new name")
    expect(page.locator(".umap-browser .datalayer")).to_contain_text("new name")
    expect(page.locator("body")).to_have_class(re.compile(".*umap-is-dirty.*"))
    with page.expect_response(re.compile(".*/datalayer/update/.*")):
        page.get_by_role("button", name="Save").click()
    saved = DataLayer.objects.last()
    assert saved.name == "new name"
    expect(page.locator(".umap-is-dirty")).to_be_hidden()


def test_can_create_new_datalayer(live_server, openmap, page, datalayer):
    """
    Test the creation and editing of a new datalayer.

    This test verifies that:
    1. A new datalayer can be created and saved.
    2. The newly created datalayer appears in the UI and is saved in the database.
    3. Editing the same datalayer updates it instead of creating a new one.
    4. The UI reflects the changes and the database is updated correctly.
    5. The 'dirty' state of the map is managed correctly during these operations.
    """

    page.goto(
        f"{live_server.url}{openmap.get_absolute_url()}?edit&onLoadPanel=databrowser"
    )
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Add a layer").click()
    page.locator('input[name="name"]').click()
    page.locator('input[name="name"]').fill("Layer A")
    expect(page.get_by_text("Layer A")).to_be_visible()
    with page.expect_response(re.compile(".*/datalayer/create/.*")):
        page.get_by_role("button", name="Save").click()
    assert DataLayer.objects.count() == 2
    saved = DataLayer.objects.last()
    assert saved.name == "Layer A"
    expect(page.locator(".umap-is-dirty")).to_be_hidden()
    # Edit again, it should not create a new datalayer
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel.right").get_by_title("Edit", exact=True).first.click()
    page.locator('input[name="name"]').click()
    page.locator('input[name="name"]').fill("Layer A with a new name")
    expect(page.get_by_text("Layer A with a new name")).to_be_visible()
    with page.expect_response(re.compile(".*/datalayer/update/.*")):
        page.get_by_role("button", name="Save").click()
    assert DataLayer.objects.count() == 2
    saved = DataLayer.objects.last()
    assert saved.name == "Layer A with a new name"
    expect(page.locator(".umap-is-dirty")).to_be_hidden()


def test_can_restore_version(live_server, openmap, page, datalayer):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    marker = page.locator(".leaflet-marker-icon")
    expect(marker).to_have_class(re.compile(".*umap-ball-icon.*"))
    marker.click(modifiers=["Shift"])
    page.get_by_text("Shape properties").click()
    page.locator("#umap-feature-shape-properties").get_by_text("Default").click()
    with page.expect_response(re.compile(".*/datalayer/update/.*")):
        page.get_by_role("button", name="Save").click()
    expect(marker).to_have_class(re.compile(".*umap-div-icon.*"))
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel.right").get_by_title("Edit", exact=True).click()
    page.get_by_text("Versions").click()
    page.get_by_title("Restore this version").last.click()
    page.get_by_role("button", name="OK").click()
    expect(marker).to_have_class(re.compile(".*umap-ball-icon.*"))


def test_can_edit_layer_on_ctrl_shift_click(live_server, openmap, page, datalayer):
    modifier = "Meta" if platform.system() == "Darwin" else "Control"
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.locator(".leaflet-marker-icon").click(modifiers=[modifier, "Shift"])
    expect(page.get_by_text("Layer properties")).to_be_visible()


def test_deleting_datalayer_should_remove_from_browser_and_layers_list(
    live_server, openmap, datalayer, page
):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    panel = page.locator(".panel.left")
    edit_panel = page.locator(".panel.right")
    page.get_by_title("Open browser").click()
    page.get_by_role("button", name="Manage layers").click()
    expect(panel.get_by_text("test datalayer")).to_be_visible()
    expect(edit_panel.get_by_text("test datalayer")).to_be_visible()
    page.locator(".panel.right").get_by_title("Delete layer").click()
    expect(panel.get_by_text("test datalayer")).to_be_hidden()
    expect(edit_panel.get_by_text("test datalayer")).to_be_hidden()


def test_deleting_datalayer_should_remove_from_caption(
    live_server, openmap, datalayer, page
):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    panel = page.locator(".panel.left")
    page.get_by_role("button", name="About").click()
    page.get_by_role("button", name="Manage layers").click()
    expect(panel.get_by_text("test datalayer")).to_be_visible()
    page.locator(".panel.right").get_by_title("Delete layer").click()
    expect(panel.get_by_text("test datalayer")).to_be_hidden()


def test_can_edit_datalayer_name_in_list(live_server, openmap, datalayer, page):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_text("test datalayer").click()
    page.get_by_text("test datalayer").fill("test datalayer foobar")
    page.get_by_role("button", name="Open browser").click()
    expect(
        page.locator(".panel.left").get_by_text("test datalayer foobar")
    ).to_be_visible()



import re
from pathlib import Path

import pytest
from playwright.sync_api import expect

from umap.models import Map, TileLayer

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db

DATALAYER_DATA = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "name": "name poly",
            },
            "id": "gyNzM",
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [11.25, 53.585984],
                        [10.151367, 52.975108],
                        [12.689209, 52.167194],
                        [14.084473, 53.199452],
                        [12.634277, 53.618579],
                        [11.25, 53.585984],
                        [11.25, 53.585984],
                    ],
                ],
            },
        },
    ],
}


@pytest.fixture
def map_with_polygon(map, live_server):
    map.settings["properties"]["zoom"] = 6
    map.settings["geometry"] = {
        "type": "Point",
        "coordinates": [8.429, 53.239],
    }
    map.edit_status = Map.ANONYMOUS
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA)
    return map


def test_can_undo_redo_map_name_change(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")

    expect(page.locator(".edit-undo")).to_be_disabled()
    expect(page.locator(".edit-redo")).to_be_disabled()
    page.get_by_title("Edit map name and caption").click()
    name_input = page.locator('.map-metadata input[name="name"]')
    expect(name_input).to_be_visible()
    name_input.click()
    name_input.press("Control+a")
    name_input.fill("New map name")
    expect(page.locator(".edit-undo")).to_be_enabled()
    expect(page.locator(".edit-redo")).to_be_disabled()
    map_name = page.locator(".umap-main-edit-toolbox .map-name")
    expect(map_name).to_have_text("New map name")
    name_input.fill("New name again")
    expect(map_name).to_have_text("New name again")

    page.locator(".edit-undo").click()
    expect(map_name).to_have_text("New map name")
    expect(page.locator(".edit-undo")).to_be_enabled()
    expect(page.locator(".edit-redo")).to_be_enabled()

    page.locator(".edit-redo").click()
    expect(map_name).to_have_text("New name again")
    expect(page.locator(".edit-undo")).to_be_enabled()
    expect(page.locator(".edit-redo")).to_be_disabled()

    page.locator(".edit-undo").click()
    expect(map_name).to_have_text("New map name")
    expect(page.locator(".edit-undo")).to_be_enabled()
    expect(page.locator(".edit-redo")).to_be_enabled()


def test_can_undo_redo_layer_color_change(
    page, map_with_polygon, live_server, tilelayer
):
    page.goto(f"{live_server.url}{map_with_polygon.get_absolute_url()}?edit")

    expect(page.locator(".edit-undo")).to_be_disabled()
    expect(page.locator(".edit-redo")).to_be_disabled()
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel").get_by_title("Edit", exact=True).click()
    page.get_by_text("Shape properties").click()
    page.locator(".umap-field-color .define").click()
    expect(page.locator(".leaflet-overlay-pane path[fill='DarkBlue']")).to_have_count(1)
    page.get_by_title("DarkRed").first.click()
    expect(page.locator(".leaflet-overlay-pane path[fill='DarkRed']")).to_have_count(1)
    expect(page.locator(".edit-undo")).to_be_enabled()
    expect(page.locator(".edit-redo")).to_be_disabled()

    page.locator(".edit-undo").click()
    expect(page.locator(".leaflet-overlay-pane path[fill='DarkBlue']")).to_have_count(1)
    expect(page.locator(".leaflet-overlay-pane path[fill='DarkRed']")).to_have_count(0)
    expect(page.locator(".edit-undo")).to_be_disabled()
    expect(page.locator(".edit-redo")).to_be_enabled()

    page.locator(".edit-redo").click()
    expect(page.locator(".leaflet-overlay-pane path[fill='DarkRed']")).to_have_count(1)
    expect(page.locator(".leaflet-overlay-pane path[fill='DarkBlue']")).to_have_count(0)
    expect(page.locator(".edit-undo")).to_be_enabled()
    expect(page.locator(".edit-redo")).to_be_disabled()


def test_can_undo_redo_tilelayer_change(live_server, page, openmap, tilelayer):
    TileLayer.objects.create(
        url_template="https://{s}.basemaps.cartocdn.com/dark_all/{z}/{x}/{y}.png",
        attribution="OSM/Carto",
        name="Black Tiles",
    )
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    old_pattern = re.compile(
        r"https://[abc]{1}.tile.openstreetmap.fr/osmfr/\d+/\d+/\d+.png"
    )
    tiles = page.locator(".leaflet-tile-pane img")
    expect(tiles.first).to_have_attribute("src", old_pattern)

    new_pattern = re.compile(
        r"https://[abcd]{1}.basemaps.cartocdn.com/dark_all/\d+/\d+/\d+.png"
    )
    page.get_by_role("button", name="Change tilelayers").click()
    page.locator("li").filter(has_text="Black Tiles").get_by_role("img").click()

    tiles = page.locator(".leaflet-tile-pane img")
    expect(tiles.first).to_have_attribute("src", new_pattern)

    page.locator(".edit-undo").click()
    tiles = page.locator(".leaflet-tile-pane img")
    expect(tiles.first).to_have_attribute("src", old_pattern)

    page.locator(".edit-redo").click()
    tiles = page.locator(".leaflet-tile-pane img")
    expect(tiles.first).to_have_attribute("src", new_pattern)


def test_can_undo_redo_marker_drag(live_server, page, tilelayer):
    page.goto(f"{live_server.url}/en/map/new")

    marker = page.locator(".leaflet-marker-icon")
    map = page.locator("#map")

    # Create a marker
    page.get_by_title("Draw a marker").click()
    map.click(position={"x": 225, "y": 225})
    expect(marker).to_have_count(1)

    # Drag marker
    old_bbox = marker.bounding_box()
    marker.first.drag_to(map, target_position={"x": 250, "y": 250})
    assert marker.bounding_box() != old_bbox

    # Undo
    page.locator(".edit-undo").click()
    assert marker.bounding_box() == old_bbox

    # Redo
    page.locator(".edit-redo").click()
    assert marker.bounding_box() != old_bbox


def test_can_undo_redo_polygon_geometry_change(live_server, page, tilelayer):
    page.goto(f"{live_server.url}/en/map/new")

    # Click on the Draw a polygon button on a new map.
    page.get_by_title("Draw a polygon").click()

    polygon = page.locator("path[fill='DarkBlue']")
    expect(polygon).to_have_count(0)

    # Click on the map, it will create a polygon.
    map = page.locator("#map")
    map.click(position={"x": 200, "y": 200})
    map.click(position={"x": 100, "y": 200})
    map.click(position={"x": 100, "y": 100})
    map.click(position={"x": 100, "y": 100})

    # It is created on peerA, and should be on peerB
    expect(polygon).to_have_count(1)
    old_bbox = polygon.bounding_box()

    edited_vertex = page.locator(".leaflet-middle-icon:nth-child(3)").first
    edited_vertex.drag_to(map, target_position={"x": 250, "y": 250})
    page.keyboard.press("Escape")

    assert polygon.bounding_box() != old_bbox

    page.locator(".edit-undo").click()
    assert polygon.bounding_box() == old_bbox

    page.locator(".edit-redo").click()
    assert polygon.bounding_box() != old_bbox


def test_can_undo_redo_marker_create(live_server, page, tilelayer):
    page.goto(f"{live_server.url}/en/map/new")

    page.get_by_title("Open Browser").click()
    marker = page.locator(".leaflet-marker-icon")
    map = page.locator("#map")

    # Create a marker
    page.get_by_title("Draw a marker").click()
    map.click(position={"x": 600, "y": 100})
    expect(marker).to_have_count(1)
    expect(page.locator(".panel .datalayer")).to_have_count(1)

    page.locator(".edit-undo").click()
    expect(marker).to_have_count(0)
    # Layer still exists
    expect(page.locator(".panel .datalayer")).to_have_count(1)

    page.locator(".edit-undo").click()
    expect(page.locator(".panel .datalayer")).to_have_count(0)

    page.locator(".edit-redo").click()
    expect(page.locator(".panel .datalayer")).to_have_count(1)

    page.locator(".edit-redo").click()
    expect(marker).to_have_count(1)


def test_undo_redo_import(live_server, page, tilelayer):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open Browser").click()

    page.get_by_title("Import data").click()
    file_input = page.locator("input[type='file']")
    with page.expect_file_chooser() as fc_info:
        file_input.click()
    file_chooser = fc_info.value
    path = Path(__file__).parent.parent / "fixtures/test_upload_data.json"
    file_chooser.set_files(path)
    page.get_by_role("button", name="Import data", exact=True).click()
    # Close the import panel
    page.keyboard.press("Escape")

    layers = page.locator(".umap-browser .datalayer")
    expect(layers).to_have_count(1)

    features_count = page.locator(".umap-browser .datalayer-counter")
    expect(features_count).to_have_text("(5)")

    page.locator(".edit-undo").click()
    expect(features_count).to_be_hidden()
    expect(layers).to_have_count(1)

    page.locator(".edit-undo").click()
    expect(layers).to_have_count(0)

    page.locator(".edit-redo").click()
    expect(layers).to_have_count(1)

    page.locator(".edit-redo").click()
    expect(features_count).to_have_text("(5)")



import os

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "umap.settings")

from django.core.asgi import get_asgi_application

from .sync.app import application as ws_application

# Initialize Django ASGI application early to ensure the AppRegistry
# is populated before importing code that may import ORM models.
django_asgi_app = get_asgi_application()


async def application(scope, receive, send):
    if scope["type"] == "http":
        await django_asgi_app(scope, receive, send)
    elif scope["type"] == "websocket":
        await ws_application(scope, receive, send)
    else:
        raise NotImplementedError(f"Unknown scope type {scope['type']}")



import platform

import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db

DATALAYER_DATA = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "name": "name poly",
            },
            "id": "gyNzM",
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [11.25, 53.585984],
                        [10.151367, 52.975108],
                        [12.689209, 52.167194],
                        [14.084473, 53.199452],
                        [12.634277, 53.618579],
                        [11.25, 53.585984],
                        [11.25, 53.585984],
                    ],
                ],
            },
        },
    ],
}


@pytest.fixture
def bootstrap(map, live_server):
    map.settings["properties"]["zoom"] = 6
    map.settings["geometry"] = {
        "type": "Point",
        "coordinates": [8.429, 53.239],
    }
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA)


def test_can_edit_on_shift_click(live_server, openmap, page, datalayer):
    modifier = "Meta" if platform.system() == "Darwin" else "Control"
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.locator(".leaflet-marker-icon").click(modifiers=[modifier, "Shift"])
    expect(page.get_by_text("Layer properties")).to_be_visible()


def test_marker_style_should_have_precedence(live_server, openmap, page, bootstrap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")

    # Change colour at layer level
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel").get_by_title("Edit", exact=True).click()
    page.get_by_text("Shape properties").click()
    page.locator(".umap-field-color .define").click()
    expect(page.locator(".leaflet-overlay-pane path[fill='DarkBlue']")).to_have_count(1)
    page.get_by_title("DarkRed").first.click()
    expect(page.locator(".leaflet-overlay-pane path[fill='DarkRed']")).to_have_count(1)

    # Now change at polygon level, it should take precedence
    page.locator("path").click(modifiers=["Shift"])
    page.get_by_text("Shape properties").click()
    page.locator("#umap-feature-shape-properties").get_by_text("define").first.click()
    page.get_by_title("GoldenRod", exact=True).first.click()
    expect(page.locator(".leaflet-overlay-pane path[fill='GoldenRod']")).to_have_count(
        1
    )

    # Now change again at layer level again, it should not change the marker color
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel").get_by_title("Edit", exact=True).click()
    page.get_by_text("Shape properties").click()
    page.locator(".umap-field-color input").click()
    page.get_by_title("DarkViolet").first.click()
    expect(page.locator(".leaflet-overlay-pane path[fill='GoldenRod']")).to_have_count(
        1
    )


def test_should_open_an_edit_toolbar_on_click(live_server, openmap, page, bootstrap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.locator("path").click()
    expect(page.get_by_role("button", name="Toggle edit mode")).to_be_visible()
    expect(page.get_by_role("button", name="Delete this feature")).to_be_visible()


def test_can_remove_stroke(live_server, openmap, page, bootstrap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    expect(page.locator(".leaflet-overlay-pane path[stroke='DarkBlue']")).to_have_count(
        1
    )
    page.locator("path").click()
    page.get_by_role("button", name="Toggle edit mode").click()
    page.get_by_text("Shape properties").click()
    page.locator(".umap-field-stroke .define").first.click()
    page.locator(".umap-field-stroke .show-on-defined label").first.click()
    expect(page.locator(".leaflet-overlay-pane path[stroke='DarkBlue']")).to_have_count(
        0
    )
    expect(page.locator(".leaflet-overlay-pane path[stroke='none']")).to_have_count(1)


def test_should_reset_style_on_cancel(live_server, openmap, page, bootstrap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.locator("path").click(modifiers=["Shift"])
    page.get_by_text("Shape properties").click()
    page.locator("#umap-feature-shape-properties").get_by_text("define").first.click()
    page.get_by_title("GoldenRod", exact=True).first.click()
    expect(page.locator(".leaflet-overlay-pane path[fill='GoldenRod']")).to_have_count(
        1
    )
    page.locator(".edit-undo").click()
    expect(page.locator(".leaflet-overlay-pane path[fill='DarkBlue']")).to_have_count(1)


def test_can_change_datalayer(live_server, openmap, page, bootstrap):
    other = DataLayerFactory(
        name="Layer 2", map=openmap, settings={"color": "GoldenRod"}
    )
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    expect(page.locator("path[fill='DarkBlue']")).to_have_count(1)
    page.locator("path").click(modifiers=["Shift"])
    page.get_by_role("combobox").select_option(other.name)
    expect(page.locator("path[fill='GoldenRod']")).to_have_count(1)



from django.conf import settings
from django.core.files.storage import FileSystemStorage
from django.core.management.base import BaseCommand

from umap.models import DataLayer, Pictogram
from umap.storage.fs import FSDataStorage


class Command(BaseCommand):
    help = "Migrate latest datalayers or pictograms from filesystem to S3."

    def add_arguments(self, parser):
        parser.add_argument(
            "--pictograms",
            help="Migrate pictograms instead of datalayers",
            action="store_true",
        )

    def handle(self, *args, **options):
        if options["pictograms"]:
            assert (
                settings.STORAGES["default"]["BACKEND"]
                == "storages.backends.s3.S3Storage"
            ), "You must configure your default storage to point to S3"
            fs_storage = FileSystemStorage()
            for picto in Pictogram.objects.all():
                fs_path = str(picto.pictogram)
                picto.pictogram.save(picto.pictogram.name, fs_storage.open(fs_path))
                if options["verbosity"] > 1:
                    print("Migrated:", fs_path)
        else:
            assert settings.UMAP_READONLY, (
                "You must run that script with a read-only uMap."
            )
            assert (
                settings.STORAGES["data"]["BACKEND"] == "umap.storage.s3.S3DataStorage"
            ), "You must configure your storages to point to S3"
            fs_storage = FSDataStorage()
            for datalayer in DataLayer.objects.all():
                geojson_fs_path = str(datalayer.geojson)
                try:
                    datalayer.geojson.save(
                        datalayer.geojson.name, fs_storage.open(geojson_fs_path)
                    )
                except FileNotFoundError as e:
                    print(e)
                    print(geojson_fs_path, "not found on disk")
                    continue
                if options["verbosity"] > 1:
                    print("Migrated:", geojson_fs_path)
                datalayer.save(force_update=True)



from django.db import models


class PublicManager(models.Manager):
    def get_queryset(self):
        return (
            super(PublicManager, self)
            .get_queryset()
            .filter(share_status=self.model.PUBLIC)
        )

    def starred_by_staff(self):
        from .models import Star, User

        staff = User.objects.filter(is_staff=True)
        stars = Star.objects.filter(by__in=staff).values("map")
        return self.get_queryset().filter(pk__in=stars)


class PrivateQuerySet(models.QuerySet):
    def for_user(self, user):
        qs = self.exclude(share_status__in=[self.model.DELETED, self.model.BLOCKED])
        teams = user.teams.all()
        qs = (
            qs.filter(owner=user)
            .union(qs.filter(editors=user))
            .union(qs.filter(team__in=teams))
        )
        return qs


class PrivateManager(models.Manager):
    def get_queryset(self):
        return PrivateQuerySet(self.model, using=self._db)

    def for_user(self, user):
        return self.get_queryset().for_user(user)



from copy import copy

from django import template
from django.conf import settings

from umap.utils import json_dumps

register = template.Library()


@register.inclusion_tag("umap/css.html")
def umap_css():
    return {"STATIC_URL": settings.STATIC_URL}


@register.inclusion_tag("umap/js.html")
def umap_js(locale=None):
    return {"STATIC_URL": settings.STATIC_URL, "locale": locale}


@register.inclusion_tag("umap/map_fragment.html")
def map_fragment(map_instance, **kwargs):
    map_settings = map_instance.preview_settings
    map_settings["properties"].update(kwargs)
    prefix = kwargs.pop("prefix", None) or "map"
    page = kwargs.pop("page", None) or ""
    unique_id = prefix + str(page) + "_" + str(map_instance.pk)
    return {
        "map_settings": json_dumps(map_settings),
        "map": map_instance,
        "unique_id": unique_id,
    }


@register.simple_tag
def tilelayer_preview(tilelayer):
    """
    Return an <img> tag with a tile of the tilelayer.
    """
    output = '<img src="{src}" alt="{alt}" title="{title}" />'
    url = tilelayer.url_template.format(s="a", z=9, x=265, y=181)
    output = output.format(src=url, alt=tilelayer.name, title=tilelayer.name)
    return output


@register.filter
def dumps(dict_):
    return json_dumps(dict_)


@register.filter
def can_delete_map(map, request):
    return map.can_delete(request)


@register.filter
def notag(s):
    return s.replace("<", "&lt;")


@register.simple_tag(takes_context=True)
def paginate_querystring(context, page):
    qs = copy(context["request"].GET)
    qs["p"] = str(page)
    return qs.urlencode()


@register.filter
def ipdb(what):
    import ipdb

    ipdb.set_trace()
    return ""


@register.filter
def addstr(arg1, arg2):
    # Necessity: https://stackoverflow.com/a/23783666
    return str(arg1) + str(arg2)



import json
import platform
import re
from pathlib import Path
from time import sleep

import pytest
from playwright.sync_api import expect

from umap.models import DataLayer

from .helpers import save_and_get_json

pytestmark = pytest.mark.django_db


def test_layers_list_is_updated(live_server, tilelayer, page):
    page.goto(f"{live_server.url}/map/new/")
    modifier = "Cmd" if platform.system() == "Darwin" else "Ctrl"
    page.get_by_role("button", name=f"Import data ({modifier}+I)").click()
    # Should work
    page.locator("[name=layer-id]").select_option(label="Import in a new layer")
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Add a layer").click()
    page.locator('input[name="name"]').click()
    page.locator('input[name="name"]').fill("foobar")
    page.wait_for_timeout(300)  # Time for the input debounce.
    page.get_by_role("button", name=f"Import data ({modifier}+I)").click()
    # Should still work
    page.locator("[name=layer-id]").select_option(label="Import in a new layer")
    # Now layer should be visible in the options
    page.locator("[name=layer-id]").select_option(label="foobar")
    expect(
        page.get_by_role("button", name="Import full map data", exact=True)
    ).to_be_hidden()
    expect(
        page.get_by_role("button", name="Link to the layer as remote data", exact=True)
    ).to_be_hidden()


def test_umap_import_from_file(live_server, tilelayer, page):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Import data").click()
    file_input = page.locator("input[type='file']")
    with page.expect_file_chooser() as fc_info:
        file_input.click()
    file_chooser = fc_info.value
    path = Path(__file__).parent.parent / "fixtures/display_on_load.umap"
    file_chooser.set_files(path)
    expect(
        page.get_by_role("button", name="Copy into the layer", exact=True)
    ).to_be_hidden()
    expect(
        page.get_by_role("button", name="Link to the layer as remote data", exact=True)
    ).to_be_hidden()
    page.get_by_role("button", name="Import data", exact=True).click()
    assert file_input.input_value()
    # Close the import panel
    page.keyboard.press("Escape")
    # Reopen
    page.get_by_title("Import data").click()
    sleep(1)  # Wait for CSS transition to happen
    assert not file_input.input_value()
    expect(page.locator(".umap-main-edit-toolbox .map-name")).to_have_text(
        "Carte sans nom"
    )
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    expect(layers).to_have_count(2)
    nonloaded = page.locator(".umap-browser .datalayer.off")
    expect(nonloaded).to_have_count(1)


def test_umap_import_from_textarea(live_server, tilelayer, page, settings):
    settings.UMAP_ALLOW_ANONYMOUS = True
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_role("button", name="Open browser").click()
    page.get_by_title("Import data").click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_data.umap"
    textarea.fill(path.read_text())
    page.locator('select[name="format"]').select_option("umap")
    page.get_by_role("button", name="Import data", exact=True).click()
    layers = page.locator(".umap-browser .datalayer")
    expect(layers).to_have_count(2)
    expect(page.locator(".umap-main-edit-toolbox .map-name")).to_have_text(
        "Imported map"
    )
    expect(page.locator(".panel.left").get_by_text("Tunnels")).to_be_visible()
    expect(page.locator(".panel.left").get_by_text("Cities")).to_be_visible()
    expect(page.locator(".leaflet-control-minimap")).to_be_visible()
    expect(
        page.locator('img[src="https://tile.openstreetmap.fr/hot/6/32/21.png"]')
    ).to_be_visible()
    # Should not have imported id, while in the file options
    assert not page.evaluate("U.MAP.properties.id")
    with page.expect_response(re.compile(r".*/datalayer/create/.*")):
        page.get_by_role("button", name="Save").click()
    assert page.evaluate("U.MAP.properties.id")


def test_import_geojson_from_textarea(tilelayer, live_server, page):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator("path")
    expect(markers).to_have_count(0)
    expect(paths).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_data.json"
    textarea.fill(path.read_text())
    page.locator('select[name="format"]').select_option("geojson")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(2)
    expect(paths).to_have_count(3)


def test_import_invalid_data(tilelayer, live_server, page):
    uncaught_errors = []
    page.on("pageerror", lambda exc: uncaught_errors.append(exc))
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator("path")
    expect(markers).to_have_count(0)
    expect(paths).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    textarea.fill("invalid data")
    for format in ["geojson", "csv", "gpx", "kml", "georss", "osm", "umap"]:
        page.locator('select[name="format"]').select_option(format)
        page.get_by_role("button", name="Import data", exact=True).click()
        assert not uncaught_errors, f"Error with format {format}"


def test_import_kml_from_textarea(tilelayer, live_server, page):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator("path")
    expect(markers).to_have_count(0)
    expect(paths).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_data.kml"
    textarea.fill(path.read_text())
    page.locator('select[name="format"]').select_option("kml")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)


def test_import_gpx_from_textarea(tilelayer, live_server, page, settings):
    settings.UMAP_ALLOW_ANONYMOUS = True
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator("path")
    expect(markers).to_have_count(0)
    expect(paths).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_data.gpx"
    textarea.fill(path.read_text())
    page.locator('select[name="format"]').select_option("gpx")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(1)
    data = save_and_get_json(page)
    assert data["features"][0]["geometry"] == {
        "coordinates": [
            [
                -121.7295456,
                45.4431641,
            ],
            [
                -121.72908,
                45.4428615,
            ],
            [
                -121.7279085,
                45.4425697,
            ],
        ],
        "type": "LineString",
    }
    assert data["features"][0]["properties"] == {
        "description": "Simple description",
        "desc": "Simple description",
        "name": "Simple path",
    }
    assert data["features"][1]["geometry"] == {
        "coordinates": [
            -121.72904,
            45.44283,
            1374,
        ],
        "type": "Point",
    }
    assert data["features"][1]["properties"] == {
        "description": "Simple description",
        "desc": "Simple description",
        "name": "Simple Point",
    }


def test_import_osm_from_textarea(tilelayer, live_server, page):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_data_osm.json"
    textarea.fill(path.read_text())
    page.locator('select[name="format"]').select_option("osm")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(2)


def test_import_csv_from_textarea(tilelayer, live_server, page):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_data.csv"
    textarea.fill(path.read_text())
    page.locator('select[name="format"]').select_option("csv")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(2)


def test_can_import_in_existing_datalayer(live_server, datalayer, page, openmap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(1)
    expect(layers).to_have_count(1)
    page.get_by_role("button", name="Edit").click()
    page.get_by_title("Import data").click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_data.csv"
    textarea.fill(path.read_text())
    page.locator('select[name="format"]').select_option("csv")
    page.locator('select[name="layer-id"]').select_option(datalayer.name)
    expect(page.locator("input[name=layer-name]")).to_be_hidden()
    page.get_by_role("button", name="Import data", exact=True).click()
    # No layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(3)


def test_can_replace_datalayer_data(live_server, datalayer, page, openmap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(1)
    expect(layers).to_have_count(1)
    page.get_by_role("button", name="Edit").click()
    page.get_by_title("Import data").click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_data.csv"
    textarea.fill(path.read_text())
    page.locator('select[name="format"]').select_option("csv")
    page.locator('select[name="layer-id"]').select_option(datalayer.name)
    page.get_by_label("Replace layer content").check()
    page.get_by_role("button", name="Import data", exact=True).click()
    # No layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(2)


def test_can_import_in_new_datalayer(live_server, datalayer, page, openmap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(1)
    expect(layers).to_have_count(1)
    page.get_by_role("button", name="Edit").click()
    page.get_by_title("Import data").click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_data.csv"
    textarea.fill(path.read_text())
    page.locator("select[name=format]").select_option("csv")
    page.locator("[name=layer-id]").select_option(label="Import in a new layer")
    page.locator("[name=layer-name]").fill("My new layer name")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A new layer has been created
    expect(layers).to_have_count(2)
    expect(markers).to_have_count(3)
    expect(page.get_by_text("My new layer name")).to_be_visible()


def test_should_remove_dot_in_property_names(live_server, page, settings, tilelayer):
    settings.UMAP_ALLOW_ANONYMOUS = True
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "geometry": {
                    "type": "Point",
                    "coordinates": [6.922931671142578, 47.481161607175736],
                },
                "type": "Feature",
                "properties": {
                    "color": "",
                    "name": "Chez Rémy",
                    "A . in the name": "",
                },
            },
            {
                "geometry": {
                    "type": "LineString",
                    "coordinates": [
                        [2.4609375, 48.88639177703194],
                        [2.48291015625, 48.76343113791796],
                        [2.164306640625, 48.719961222646276],
                    ],
                },
                "type": "Feature",
                "properties": {"color": "", "name": "Périf", "with a dot.": ""},
            },
        ],
    }
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Import data").click()
    textarea = page.locator(".umap-import textarea")
    textarea.fill(json.dumps(data))
    page.locator('select[name="format"]').select_option("geojson")
    page.get_by_role("button", name="Import data", exact=True).click()
    with page.expect_response(re.compile(r".*/datalayer/create/.*")):
        page.get_by_role("button", name="Save").click()
    datalayer = DataLayer.objects.last()
    saved_data = json.loads(Path(datalayer.geojson.path).read_text())
    assert saved_data["features"][0]["properties"] == {
        "color": "",
        "name": "Chez Rémy",
        "A _ in the name": "",
    }
    assert saved_data["features"][1]["properties"] == {
        "color": "",
        "name": "Périf",
        "with a dot_": "",
    }


def test_import_geometry_collection(live_server, page, tilelayer):
    data = {
        "type": "GeometryCollection",
        "geometries": [
            {"type": "Point", "coordinates": [-80.6608, 35.0493]},
            {
                "type": "Polygon",
                "coordinates": [
                    [
                        [-80.6645, 35.0449],
                        [-80.6634, 35.0460],
                        [-80.6625, 35.0455],
                        [-80.6638, 35.0442],
                        [-80.6645, 35.0449],
                    ]
                ],
            },
            {
                "type": "LineString",
                "coordinates": [
                    [-80.66237, 35.05950],
                    [-80.66269, 35.05926],
                    [-80.66284, 35.05893],
                    [-80.66308, 35.05833],
                    [-80.66385, 35.04387],
                    [-80.66303, 35.04371],
                ],
            },
        ],
    }
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator("path")
    expect(markers).to_have_count(0)
    expect(paths).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    textarea.fill(json.dumps(data))
    page.locator('select[name="format"]').select_option("geojson")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)


def test_import_geometry_collection_in_feature(live_server, page, tilelayer):
    data = {
        "type": "Feature",
        "properties": {"name": "foobar"},
        "geometry": {
            "type": "GeometryCollection",
            "geometries": [
                {"type": "Point", "coordinates": [-80.6608, 35.0493]},
                {
                    "type": "Polygon",
                    "coordinates": [
                        [
                            [-80.6645, 35.0449],
                            [-80.6634, 35.0460],
                            [-80.6625, 35.0455],
                            [-80.6638, 35.0442],
                            [-80.6645, 35.0449],
                        ]
                    ],
                },
                {
                    "type": "LineString",
                    "coordinates": [
                        [-80.66237, 35.05950],
                        [-80.66269, 35.05926],
                        [-80.66284, 35.05893],
                        [-80.66308, 35.05833],
                        [-80.66385, 35.04387],
                        [-80.66303, 35.04371],
                    ],
                },
            ],
        },
    }
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator("path")
    expect(markers).to_have_count(0)
    expect(paths).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    textarea.fill(json.dumps(data))
    page.locator('select[name="format"]').select_option("geojson")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)
    # Geometries are treated as separate features.
    expect(page.get_by_text("foobar")).to_have_count(3)


def test_import_multipolygon(live_server, page, tilelayer):
    data = {
        "type": "Feature",
        "properties": {"name": "Some states"},
        "geometry": {
            "type": "MultiPolygon",
            "coordinates": [
                [
                    [[-109, 36], [-109, 40], [-102, 37], [-109, 36]],
                    [[-108, 39], [-107, 37], [-104, 37], [-108, 39]],
                ],
                [[[-119, 42], [-120, 39], [-114, 41], [-119, 42]]],
            ],
        },
    }
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    paths = page.locator("path")
    expect(paths).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    textarea.fill(json.dumps(data))
    page.locator('select[name="format"]').select_option("geojson")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(paths).to_have_count(1)


def test_import_multipolyline(live_server, page, tilelayer):
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {},
                "geometry": {
                    "type": "MultiLineString",
                    "coordinates": [[[-108, 46], [-113, 43]], [[-112, 45], [-115, 44]]],
                },
            }
        ],
    }
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    paths = page.locator("path")
    expect(paths).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    textarea.fill(json.dumps(data))
    page.locator('select[name="format"]').select_option("geojson")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(paths).to_have_count(1)


def test_should_not_import_empty_coordinates(live_server, page, tilelayer):
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"name": "An empty polygon"},
                "geometry": {
                    "type": "Polygon",
                    "coordinates": [],
                },
            },
            {
                "type": "Feature",
                "properties": {"name": "An empty linestring"},
                "geometry": {
                    "type": "LineString",
                    "coordinates": [],
                },
            },
            {
                "type": "Feature",
                "properties": {"name": "An empty point"},
                "geometry": {
                    "type": "Point",
                    "coordinates": [],
                },
            },
            {
                "geometry": {
                    "type": "Point",
                    "coordinates": [6.922931671142578, 47.481161607175736],
                },
                "type": "Feature",
                "properties": {"name": "A point"},
            },
            {
                "geometry": {
                    "type": "LineString",
                    "coordinates": [
                        [2.4609375, 48.88639177703194],
                        [2.48291015625, 48.76343113791796],
                        [2.164306640625, 48.719961222646276],
                    ],
                },
                "type": "Feature",
                "properties": {
                    "name": "A linestring",
                },
            },
        ],
    }
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    page.get_by_title("Import data").click()
    textarea = page.locator(".umap-import textarea")
    textarea.fill(json.dumps(data))
    page.locator('select[name="format"]').select_option("geojson")
    page.get_by_role("button", name="Import data", exact=True).click()
    page.locator("#map").click(button="right")
    page.get_by_role("button", name="browse data").click()
    expect(page.locator("li.feature")).to_have_count(2)


def test_import_csv_without_valid_latlon_headers(tilelayer, live_server, page):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    page.get_by_title("Import data").click()
    textarea = page.locator(".umap-import textarea")
    textarea.fill("a,b,c\n12.23,48.34,mypoint\n12.23,48.34,mypoint2")
    page.locator('select[name="format"]').select_option("csv")
    page.get_by_role("button", name="Import data", exact=True).click()
    # FIXME do not create a layer
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(0)
    expect(page.locator('umap-alert div[data-level="error"]')).to_be_visible()


def test_import_csv_with_commas_in_latlon(tilelayer, live_server, page, settings):
    settings.UMAP_ALLOW_ANONYMOUS = True
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    page.get_by_title("Import data").click()
    textarea = page.locator(".umap-import textarea")
    textarea.fill("lat;lon;foobar\n12,24;48,34;mypoint\n12,23;48,35;mypoint2")
    page.locator('select[name="format"]').select_option("csv")
    page.get_by_role("button", name="Import data", exact=True).click()
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(2)
    with page.expect_response(re.compile(r".*/datalayer/create/.*")):
        page.get_by_role("button", name="Save").click()
    datalayer = DataLayer.objects.last()
    saved_data = json.loads(Path(datalayer.geojson.path).read_text())
    assert saved_data["features"][0]["geometry"] == {
        "coordinates": [
            48.35,
            12.23,
        ],
        "type": "Point",
    }
    assert saved_data["features"][1]["geometry"] == {
        "coordinates": [
            48.34,
            12.24,
        ],
        "type": "Point",
    }


def test_create_remote_data(page, live_server, tilelayer):
    def handle(route):
        route.fulfill(
            json={
                "type": "FeatureCollection",
                "features": [
                    {
                        "type": "Feature",
                        "properties": {},
                        "geometry": {
                            "type": "Point",
                            "coordinates": [4.3375, 51.2707],
                        },
                    }
                ],
            }
        )

    # Intercept the route to the proxy
    page.route("*/**/ajax-proxy/**", handle)
    page.goto(f"{live_server.url}/map/new/")
    expect(page.locator(".leaflet-marker-icon")).to_be_hidden()
    page.get_by_role("button", name="Import data").click()
    page.get_by_placeholder("Provide an URL here").click()
    page.get_by_placeholder("Provide an URL here").fill("https://remote.org/data.json")
    page.locator("[name=format]").select_option("geojson")
    page.get_by_role("radio", name="Link to the layer as remote data").click()
    page.get_by_role("button", name="Import data", exact=True).click()
    expect(page.locator(".leaflet-marker-icon")).to_be_visible()
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.locator("summary").filter(has_text="Remote data").click()
    expect(page.locator('.panel input[name="url"]')).to_have_value(
        "https://remote.org/data.json"
    )


def test_create_remote_data_from_umap_backup(page, live_server, tilelayer):
    def handle(route):
        route.fulfill(
            json={
                "type": "FeatureCollection",
                "features": [
                    {
                        "type": "Feature",
                        "properties": {},
                        "geometry": {
                            "type": "Point",
                            "coordinates": [4.3375, 51.2707],
                        },
                    }
                ],
            }
        )

    page.route("https://remote.org/data.json", handle)
    page.goto(f"{live_server.url}/map/new/")
    expect(page.locator(".leaflet-marker-icon")).to_be_hidden()
    page.get_by_title("Import data").click()
    file_input = page.locator("input[type='file']")
    with page.expect_file_chooser() as fc_info:
        file_input.click()
    file_chooser = fc_info.value
    path = Path(__file__).parent.parent / "fixtures/remote_data.umap"
    file_chooser.set_files(path)
    page.get_by_role("button", name="Import data", exact=True).click()
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    expect(layers).to_have_count(1)
    expect(page.locator(".leaflet-marker-icon")).to_be_visible()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.locator("summary").filter(has_text="Remote data").click()
    expect(page.locator('.panel input[name="url"]')).to_have_value(
        "https://remote.org/data.json"
    )


def test_import_geojson_from_url(page, live_server, tilelayer):
    def handle(route):
        route.fulfill(
            json={
                "type": "FeatureCollection",
                "features": [
                    {
                        "type": "Feature",
                        "properties": {},
                        "geometry": {
                            "type": "Point",
                            "coordinates": [4.3375, 51.2707],
                        },
                    }
                ],
            }
        )

    # Intercept the route
    page.route("https://remote.org/data.json", handle)
    page.goto(f"{live_server.url}/map/new/")
    expect(page.locator(".leaflet-marker-icon")).to_be_hidden()
    page.get_by_role("button", name="Import data").click()
    page.get_by_placeholder("Provide an URL here").click()
    page.get_by_placeholder("Provide an URL here").fill("https://remote.org/data.json")
    page.locator("[name=format]").select_option("geojson")
    page.get_by_role("radio", name="Copy into the layer").click()
    page.get_by_role("button", name="Import data", exact=True).click()
    expect(page.locator(".leaflet-marker-icon")).to_be_visible()
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.locator("summary").filter(has_text="Remote data").click()
    expect(page.locator('.panel input[name="url"]')).to_have_value("")


def test_overpass_import_with_bbox(page, live_server, tilelayer, settings):
    settings.UMAP_IMPORTERS = {
        "overpass": {"url": "https://my.overpass.io/interpreter"}
    }
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_role("button", name="Import data").click()
    page.get_by_role("button", name="Import helpers").click()
    page.get_by_role("button", name="Overpass").click()
    page.get_by_placeholder("amenity=drinking_water").fill("building")
    page.get_by_role("button", name="Choose this data").click()
    expect(page.get_by_placeholder("Provide an URL here")).to_have_value(
        "https://my.overpass.io/interpreter?data=[out:json];nwr[building]({south},{west},{north},{east});out geom;"
    )


def test_overpass_import_retains_boundary(page, live_server, tilelayer, settings):
    settings.UMAP_IMPORTERS = {
        "overpass": {
            "url": "https://my.overpass.io/interpreter",
            "searchUrl": "https://foobar.io/api?q={q}",
        }
    }

    def handle(route):
        route.fulfill(
            json={
                "features": [
                    {
                        "geometry": {
                            "coordinates": [3.2394035, 48.4149956],
                            "type": "Point",
                        },
                        "type": "Feature",
                        "properties": {
                            "osm_type": "R",
                            "osm_id": 1393025,
                            "extent": [3.2290211, 48.4268302, 3.2623032, 48.4041636],
                            "country": "France",
                            "osm_key": "place",
                            "countrycode": "FR",
                            "osm_value": "village",
                            "postcode": "77480",
                            "name": "Bray-sur-Seine",
                            "county": "Seine-et-Marne",
                            "state": "Île-de-France",
                            "type": "city",
                        },
                    }
                ],
                "type": "FeatureCollection",
            }
        )

    # Intercept the route
    page.route(re.compile("https://foobar.io/api.*"), handle)
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_role("button", name="Import data").click()
    page.get_by_role("button", name="Import helpers").click()
    page.get_by_role("button", name="Overpass").click()
    page.get_by_placeholder("amenity=drinking_water").fill("building")
    page.get_by_placeholder("Type area name, or let empty").click()
    page.get_by_placeholder("Type area name, or let empty").press_sequentially("bray")
    page.get_by_text("Bray-sur-Seine, Seine-et-Marne, Île-de-France, France").click()
    expect(page.locator("#area")).to_contain_text(
        "Bray-sur-Seine, Seine-et-Marne, Île-de-France, France"
    )
    page.get_by_role("button", name="Choose this data").click()
    expect(page.get_by_placeholder("Provide an URL here")).to_have_value(
        "https://my.overpass.io/interpreter?data=[out:json];nwr[building](area:3601393025);out geom;"
    )
    page.get_by_role("button", name="Import helpers").click()
    page.get_by_role("button", name="Overpass").click()
    expect(page.locator("#area")).to_contain_text(
        "Bray-sur-Seine, Seine-et-Marne, Île-de-France, France"
    )


def test_import_from_datasets(page, live_server, tilelayer, settings):
    settings.UMAP_IMPORTERS = {
        "datasets": {
            "choices": [
                {
                    "url": "https://remote.org/data.json",
                    "label": "Good data",
                    "format": "geojson",
                }
            ]
        }
    }

    def handle(route):
        route.fulfill(
            json={
                "type": "FeatureCollection",
                "features": [
                    {
                        "type": "Feature",
                        "properties": {},
                        "geometry": {
                            "type": "Point",
                            "coordinates": [4.3375, 51.2707],
                        },
                    }
                ],
            }
        )

    # Intercept the route
    page.route("https://remote.org/data.json", handle)
    page.goto(f"{live_server.url}/map/new/")
    expect(page.locator(".leaflet-marker-icon")).to_be_hidden()
    page.get_by_title("Import data").click()
    page.get_by_role("button", name="Import helpers").click()
    page.get_by_role("button", name="Datasets").click()
    page.get_by_role("dialog").get_by_role("combobox").select_option(
        "https://remote.org/data.json"
    )
    page.get_by_role("button", name="Choose this dataset").click()
    page.get_by_label("Copy into the layer").check()
    page.get_by_role("button", name="Import data", exact=True).click()
    expect(page.locator(".leaflet-marker-icon")).to_be_visible()
    page.get_by_role("button", name="Open browser").click()
    expect(page.locator("summary").get_by_text("Good data")).to_be_visible()


def test_import_osm_relation(tilelayer, live_server, page):
    # Overpass query used for this data:
    # [out:json][timeout:25];(relation(id:15612202)%20;);out%20geom;
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    paths = page.locator("path")
    expect(paths).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    file_path = Path(__file__).parent.parent / "fixtures/test_import_osm_relation.json"
    textarea.fill(file_path.read_text())
    page.locator('select[name="format"]').select_option("osm")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer and one path has been created
    expect(layers).to_have_count(1)
    expect(paths).to_have_count(1)


def test_import_georss_from_textarea(tilelayer, live_server, page):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Open browser").click()
    layers = page.locator(".umap-browser .datalayer")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(0)
    expect(layers).to_have_count(0)
    button = page.get_by_title("Import data")
    expect(button).to_be_visible()
    button.click()
    textarea = page.locator(".umap-import textarea")
    path = Path(__file__).parent.parent / "fixtures/test_upload_georss.xml"
    textarea.fill(path.read_text())
    page.locator('select[name="format"]').select_option("georss")
    page.get_by_role("button", name="Import data", exact=True).click()
    # A layer has been created
    expect(layers).to_have_count(1)
    expect(markers).to_have_count(1)


def test_import_from_multiple_files(live_server, page, tilelayer):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Import data").click()
    file_input = page.locator("input[type='file']")
    with page.expect_file_chooser() as fc_info:
        file_input.click()
    file_chooser = fc_info.value
    FIXTURES = Path(__file__).parent.parent / "fixtures"
    paths = [
        FIXTURES / "test_upload_data.json",
        FIXTURES / "test_upload_simple_marker.json",
    ]
    file_chooser.set_files(paths)
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(0)
    page.get_by_role("button", name="Import data", exact=True).click()
    # Two in one file, one in the other
    expect(markers).to_have_count(3)


def test_umap_import_with_iconurl(live_server, tilelayer, page):
    page.goto(f"{live_server.url}/map/new/")
    page.get_by_title("Import data").click()
    file_input = page.locator("input[type='file']")
    with page.expect_file_chooser() as fc_info:
        file_input.click()
    file_chooser = fc_info.value
    path = Path(__file__).parent.parent / "fixtures/test_upload_data_with_iconurl.umap"
    file_chooser.set_files(path)
    page.get_by_role("button", name="Import data", exact=True).click()
    expect(
        page.locator(
            'img[src="https://umap.incubateur.anct.gouv.fr/uploads/pictogram/car-24.png"]'
        )
    ).to_have_count(2)



import pytest
from playwright.sync_api import expect

pytestmark = pytest.mark.django_db


def test_should_not_render_any_control(live_server, tilelayer, page, map):
    map.settings["properties"]["onLoadPanel"] = "databrowser"
    map.settings["properties"]["miniMap"] = True
    map.settings["properties"]["captionBar"] = True
    map.save()
    # Make sure those controls are visible in normal view
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(page.locator(".leaflet-control-minimap")).to_be_visible()
    expect(page.locator(".umap-browser")).to_be_visible()
    expect(page.locator(".umap-caption-bar")).to_be_visible()
    expect(page.locator(".leaflet-control-zoom")).to_be_visible()
    expect(page.locator(".leaflet-control-attribution")).to_be_visible()

    # Now load home page to have the list view
    page.goto(live_server.url)
    map_el = page.locator(".map_fragment")
    expect(map_el).to_be_visible()
    expect(map_el.locator(".leaflet-control-minimap")).to_be_hidden()
    expect(map_el.locator(".umap-browser")).to_be_hidden()
    expect(map_el.locator(".umap-caption-bar")).to_be_hidden()
    expect(map_el.locator(".leaflet-control-zoom")).to_be_hidden()
    expect(map_el.locator(".leaflet-control-attribution")).to_be_hidden()



import pytest
from django.core.management import call_command

from umap.models import Map, Team

from .base import MapFactory

pytestmark = pytest.mark.django_db


def test_switch_user(user, user2, map, team):
    user_owned = MapFactory(owner=user)
    user2_owned = MapFactory(owner=user2)
    map.editors.add(user)
    map.save()
    team.users.add(user)
    team.save()
    call_command("switch_user", user.username, user2.username, "--dry-run")
    assert Map.objects.get(pk=user_owned.pk).owner == user
    assert Map.objects.get(pk=user2_owned.pk).owner == user2
    assert user in Map.objects.get(pk=map.pk).editors.all()
    assert user2 not in Map.objects.get(pk=map.pk).editors.all()
    assert user in Team.objects.get(pk=team.pk).users.all()
    assert user2 not in Team.objects.get(pk=team.pk).users.all()
    call_command("switch_user", user.username, user2.username)
    assert Map.objects.get(pk=user_owned.pk).owner == user2
    assert Map.objects.get(pk=user2_owned.pk).owner == user2
    assert user not in Map.objects.get(pk=map.pk).editors.all()
    assert user2 in Map.objects.get(pk=map.pk).editors.all()
    assert user not in Team.objects.get(pk=team.pk).users.all()
    assert user2 in Team.objects.get(pk=team.pk).users.all()



import re


def test_reseting_map_would_remove_from_save_queue(
    live_server, openmap, page, datalayer
):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.get_by_role("button", name="Edit map name and caption").click()
    requests = []

    def register_request(request):
        if request.url.endswith(".png"):
            return
        requests.append((request.method, request.url))

    page.on("request", register_request)
    page.locator('input[name="name"]').click()
    page.locator('input[name="name"]').fill("new name")
    page.locator(".edit-undo").click()
    page.wait_for_timeout(500)
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.locator('input[name="name"]').click()
    page.locator('input[name="name"]').fill("new datalayer name")
    page.wait_for_timeout(300)  # Time of the Input debounce
    with page.expect_response(re.compile(".*/datalayer/update/.*")):
        page.get_by_role("button", name="Save", exact=True).click()
    assert len(requests) == 1
    assert requests == [
        (
            "POST",
            f"{live_server.url}/en/map/{openmap.pk}/datalayer/update/{datalayer.pk}/",
        ),
    ]



import json
from pathlib import Path

import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


def test_heatmap(map, live_server, page):
    path = Path(__file__).parent.parent / "fixtures/heatmap_data.json"
    data = json.loads(path.read_text())
    DataLayerFactory(data=data, map=map)
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(page.locator("canvas.leaflet-heatmap-layer")).to_be_visible()


def test_can_create_heatmap_after_import(live_server, page, tilelayer, settings):
    settings.UMAP_ALLOW_ANONYMOUS = True
    page.goto(f"{live_server.url}/en/map/new")
    page.get_by_title("Import data").click()
    file_input = page.locator("input[type='file']")
    with page.expect_file_chooser() as fc_info:
        file_input.click()
    file_chooser = fc_info.value
    path = Path(__file__).parent.parent / "fixtures/heatmap_data.json"
    file_chooser.set_files(path)
    page.get_by_role("button", name="Import data", exact=True).click()
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Edit", exact=True).click()
    page.get_by_role("combobox").select_option("Heat")
    page.get_by_role("button", name="Save draft").click()
    expect(page.locator("canvas.leaflet-heatmap-layer")).to_be_visible()

    # Test we can delete it and save
    page.get_by_text("Advanced actions").click()
    page.get_by_role("button", name="Delete").click()
    page.get_by_role("button", name="Save draft").click()
    expect(page.locator("canvas.leaflet-heatmap-layer")).to_be_hidden()



from django.conf import settings
from django.contrib import messages
from django.contrib.auth import BACKEND_SESSION_KEY
from django.core.exceptions import MiddlewareNotUsed
from django.http import (
    HttpResponseForbidden,
    HttpResponseRedirect,
)
from django.urls import reverse
from django.utils.translation import gettext as _


def readonly_middleware(get_response):
    if not settings.UMAP_READONLY:
        raise MiddlewareNotUsed

    def middleware(request):
        if request.method not in ["GET", "OPTIONS", "HEAD"]:
            return HttpResponseForbidden(_("Site is readonly for maintenance"))

        return get_response(request)

    return middleware


def deprecated_auth_backend(get_response):
    def middleware(request):
        backend = request.session.get(BACKEND_SESSION_KEY)
        if backend in settings.DEPRECATED_AUTHENTICATION_BACKENDS:
            name = backend.split(".")[-1]
            messages.error(
                request,
                _(
                    "Using “%(name)s” to authenticate is deprecated and will be "
                    "removed soon. "
                    "Please configure another provider below before losing access "
                    "to your account and maps. Then, please logout and login "
                    "again with the new provider."
                )
                % {"name": name},
            )
            if "/map/" in request.path:
                return HttpResponseRedirect(reverse("user_profile"))

        return get_response(request)

    return middleware



import re

import pytest
from playwright.sync_api import expect

from umap.models import Map


def test_page_title(page, live_server):
    page.goto(live_server.url)
    expect(page).to_have_title("uMap - Online map creator")


@pytest.mark.parametrize(
    "lang,link_name,link_url",
    [("fr", "Créer une carte", "/fr/map/new/"), ("en", "Create a map", "/en/map/new/")],
)
def test_create_map_link(page, live_server, lang, link_name, link_url):
    page.goto(f"{live_server.url}/{lang}/")
    create_map_button = page.locator("header nav a.button")
    expect(create_map_button).to_have_text(link_name)
    expect(create_map_button).to_have_attribute("href", link_url)


def test_create_map_with_cursor(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/en/map/new/")

    # Click on the Draw a marker button on a new map.
    create_marker_link = page.locator(".umap-edit-bar ").get_by_title("Draw a marker")
    create_marker_link.click()

    # Check no marker is present by default.
    marker_pane_children = page.locator(".leaflet-marker-pane > div")
    expect(marker_pane_children).to_have_count(0)

    # Click on the map, it will place a marker at the given position.
    map = page.locator("#map")
    map.click(position={"x": 200, "y": 200})
    expect(marker_pane_children).to_have_count(1)
    expect(marker_pane_children).to_have_attribute(
        "style",
        (
            "margin-left: -16px; "
            "margin-top: -40px; "
            "transform: translate3d(200px, 200px, 0px); "
            "z-index: 200;"
        ),
    )


def test_cannot_put_script_tag_in_datalayer_name_or_description(
    openmap, live_server, page, tilelayer
):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}")
    page.get_by_role("button", name="Edit").click()
    page.get_by_role("button", name="Manage layers").click()
    page.get_by_role("button", name="Add a layer").click()
    page.locator('input[name="name"]').click()
    page.locator('input[name="name"]').fill('<script>alert("attack")</script>')
    page.locator(".umap-field-description textarea").click()
    page.locator(".umap-field-description textarea").fill(
        '<p>before <script>alert("attack")</script> after</p>'
    )
    page.wait_for_timeout(300)  # Wait for debounce
    with page.expect_response(re.compile(".*/datalayer/create/.*")):
        page.get_by_role("button", name="Save").click()
    page.get_by_role("button", name="About").click()
    # Title should contain raw HTML (we are using textContent)
    expect(page.get_by_text('<script>alert("attack")</script>')).to_be_visible()
    # Description should contain escaped HTML
    expect(page.get_by_text("before after")).to_be_visible()

    # Reload the map
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}")
    page.get_by_role("button", name="About").click()
    expect(page.get_by_text('<script>alert("attack")</script>')).to_be_visible()
    expect(page.get_by_text("before after")).to_be_visible()


def test_login_from_map_page(live_server, page, tilelayer, settings, user, context):
    settings.ENABLE_ACCOUNT_LOGIN = True
    assert Map.objects.count() == 0
    page.goto(f"{live_server.url}/en/map/new/")
    with (
        page.expect_response(re.compile(r".*/map/create/")),
        context.expect_page() as login_page_info,
    ):
        page.get_by_role("button", name="Save").click()
    assert Map.objects.count() == 0
    login_page = login_page_info.value
    expect(login_page).to_have_title("Login - Online map creator")
    login_page.get_by_placeholder("Username").fill(user.username)
    login_page.get_by_placeholder("Password").fill("123123")
    with page.expect_response(re.compile(r".*/map/create/")):
        login_page.locator('#login_form input[type="submit"]').click()
    # Login page should be closed
    page.wait_for_timeout(500)  # Seems needed from time to time…
    assert len(context.pages) == 1
    # Save should have proceed
    assert Map.objects.count() == 1
    # Use name should now appear on the header toolbar
    expect(page.get_by_role("button", name="Joe")).to_be_visible()



from playwright.sync_api import expect


def test_home_control_is_hidden(live_server, map, tilelayer, page):
    body = f"""
    <html>
        <head></head>
        <body>
            <iframe width="100%" height="300px" frameborder="0" allowfullscreen allow="geolocation"
            src="{map.get_absolute_url()}?scaleControl=false&miniMap=false&scrollWheelZoom=false&zoomControl=true&editMode=disabled&moreControl=true&searchControl=null&tilelayersControl=null&embedControl=null&datalayersControl=true&onLoadPanel=caption&captionBar=false&captionMenus=true"></iframe>
        </body>
    </html>
    """

    def handle(route):
        route.fulfill(body=body)

    url = f"{live_server.url}/test-iframe"
    # Intercept the route
    page.route(url, handle)

    page.goto(url)
    expect(
        page.locator("iframe").content_frame.get_by_role("link", name="Home logo")
    ).to_be_hidden()



import json
import uuid

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.gis.db import models
from django.contrib.postgres.fields import ArrayField
from django.core.files.base import File
from django.core.files.storage import storages
from django.core.signing import Signer
from django.urls import reverse
from django.utils.functional import classproperty
from django.utils.translation import gettext_lazy as _

from .managers import PrivateManager, PublicManager
from .utils import _urls_for_js


# Did not find a clean way to do this in Django
# - creating a Proxy model would mean replacing get_user_model by this proxy model
#   in every template
# - extending User model would mean a non trivial migration
def display_name(self):
    return settings.USER_DISPLAY_NAME.format(**self.__dict__)


def get_user_url(self):
    identifier = getattr(self, settings.USER_URL_FIELD)
    return reverse(settings.USER_MAPS_URL, kwargs={"identifier": identifier})


def get_user_stars_url(self):
    identifier = getattr(self, settings.USER_URL_FIELD)
    return reverse("user_stars", kwargs={"identifier": identifier})


def get_user_metadata(self):
    return {
        "id": self.pk,
        "name": str(self),
        "url": self.get_url(),
    }


User.add_to_class("__str__", display_name)
User.add_to_class("get_url", get_user_url)
User.add_to_class("get_stars_url", get_user_stars_url)
User.add_to_class("get_metadata", get_user_metadata)


def get_default_share_status():
    return settings.UMAP_DEFAULT_SHARE_STATUS or Map.DRAFT


def get_default_edit_status():
    return settings.UMAP_DEFAULT_EDIT_STATUS or Map.OWNER


class Team(models.Model):
    name = models.CharField(
        max_length=200, verbose_name=_("name"), unique=True, blank=False, null=False
    )
    description = models.TextField(blank=True, null=True, verbose_name=_("description"))
    users = models.ManyToManyField(User, related_name="teams")

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.name

    def get_url(self):
        return reverse("team_maps", kwargs={"pk": self.pk})

    def get_metadata(self):
        return {"id": self.pk, "name": self.name, "url": self.get_url()}


class NamedModel(models.Model):
    name = models.CharField(max_length=200, verbose_name=_("name"))

    class Meta:
        abstract = True
        ordering = ("name",)

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.name


def get_default_licence():
    """
    Returns a default Licence, creates it if it doesn't exist.
    Needed to prevent a licence deletion from deleting all the linked
    maps.
    """
    return Licence.objects.get_or_create(
        # can't use ugettext_lazy for database storage, see #13965
        name=getattr(settings, "UMAP_DEFAULT_LICENCE_NAME", "No licence set")
    )[0]


class Licence(NamedModel):
    """
    The licence one map is published on.
    """

    details = models.URLField(
        verbose_name=_("details"),
        help_text=_("Link to a page where the licence is detailed."),
    )

    @property
    def json(self):
        return {"name": self.name, "url": self.details}


class TileLayer(NamedModel):
    url_template = models.CharField(
        max_length=400, help_text=_("URL template using OSM tile format")
    )
    minZoom = models.IntegerField(default=0)
    maxZoom = models.IntegerField(default=18)
    attribution = models.CharField(max_length=300)
    rank = models.SmallIntegerField(
        blank=True, null=True, help_text=_("Order of the tilelayers in the edit box")
    )
    # See https://wiki.openstreetmap.org/wiki/TMS#The_Y_coordinate
    tms = models.BooleanField(default=False)

    @property
    def json(self):
        return dict(
            (field.name, getattr(self, field.name)) for field in self._meta.fields
        )

    @classmethod
    def get_default(cls):
        """
        Returns the default tile layer (used for a map when no layer is set).
        """
        return cls.objects.order_by("rank")[0]  # FIXME, make it administrable

    @classmethod
    def get_list(cls):
        default = cls.get_default()
        l = []
        for t in cls.objects.all():
            fields = t.json
            if default and default.pk == t.pk:
                fields["selected"] = True
            l.append(fields)
        return l

    class Meta:
        ordering = ("rank", "name")


class Map(NamedModel):
    """
    A single thematical map.
    """

    ANONYMOUS = 1
    COLLABORATORS = 2
    OWNER = 3
    DRAFT = 0
    PUBLIC = 1
    OPEN = 2
    PRIVATE = 3
    BLOCKED = 9
    DELETED = 99
    ANONYMOUS_EDIT_STATUS = (
        (OWNER, _("Only editable with secret edit link")),
        (ANONYMOUS, _("Everyone can edit")),
    )
    EDIT_STATUS = (
        (ANONYMOUS, _("Everyone")),
        (COLLABORATORS, _("Editors and team only")),
        (OWNER, _("Owner only")),
    )
    ANONYMOUS_SHARE_STATUS = (
        (DRAFT, _("Draft (private)")),
        (PUBLIC, _("Everyone (public)")),
    )
    SHARE_STATUS = ANONYMOUS_SHARE_STATUS + (
        (OPEN, _("Anyone with link")),
        (PRIVATE, _("Editors and team only")),
        (BLOCKED, _("Blocked")),
        (DELETED, _("Deleted")),
    )
    slug = models.SlugField(db_index=True)
    center = models.PointField(geography=True, verbose_name=_("center"))
    zoom = models.IntegerField(default=7, verbose_name=_("zoom"))
    locate = models.BooleanField(
        default=False, verbose_name=_("locate"), help_text=_("Locate user on load?")
    )
    licence = models.ForeignKey(
        Licence,
        help_text=_("Choose the map licence."),
        verbose_name=_("licence"),
        on_delete=models.SET_DEFAULT,
        default=get_default_licence,
    )
    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)
    owner = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        blank=True,
        null=True,
        related_name="owned_maps",
        verbose_name=_("owner"),
        on_delete=models.PROTECT,
    )
    editors = models.ManyToManyField(
        settings.AUTH_USER_MODEL, blank=True, verbose_name=_("editors")
    )
    team = models.ForeignKey(
        Team,
        blank=True,
        null=True,
        verbose_name=_("team"),
        on_delete=models.SET_NULL,
    )
    edit_status = models.SmallIntegerField(
        choices=EDIT_STATUS,
        default=get_default_edit_status,
        verbose_name=_("edit status"),
    )
    share_status = models.SmallIntegerField(
        choices=SHARE_STATUS,
        default=get_default_share_status,
        verbose_name=_("share status"),
    )
    settings = models.JSONField(
        blank=True, null=True, verbose_name=_("settings"), default=dict
    )
    tags = ArrayField(models.CharField(max_length=200), blank=True, default=list)
    is_template = models.BooleanField(
        default=False,
        verbose_name=_("save as template"),
        help_text=_("This map is a template map."),
    )

    objects = models.Manager()
    public = PublicManager()
    private = PrivateManager()

    @property
    def description(self):
        try:
            return self.settings["properties"]["description"]
        except KeyError:
            return ""

    @property
    def datalayers(self):
        return self.datalayer_set.filter(share_status=DataLayer.INHERIT).all()

    @property
    def preview_settings(self):
        layers = self.datalayers
        datalayer_data = [c.metadata() for c in layers]
        map_settings = self.settings
        if "properties" not in map_settings:
            map_settings["properties"] = {}
        map_settings["properties"].update(
            {
                "tilelayers": [TileLayer.get_default().json],
                "datalayers": datalayer_data,
                "urls": _urls_for_js(),
                "STATIC_URL": settings.STATIC_URL,
                "editMode": "disabled",
                "hash": False,
                "scrollWheelZoom": False,
                "noControl": True,
                "id": self.pk,
                "schema": self.extra_schema,
                "slideshow": {},
                "defaultLabelKeys": settings.UMAP_LABEL_KEYS,
            }
        )
        return map_settings

    def move_to_trash(self):
        self.share_status = Map.DELETED
        self.save()

    def delete(self, **kwargs):
        # Explicitely call datalayers.delete, so we can deal with removing files
        # (the cascade delete would not call the model delete method)
        # Use datalayer_set so to get also the deleted ones.
        for datalayer in self.datalayer_set.all():
            datalayer.delete()
        return super().delete(**kwargs)

    def generate_umapjson(self, request, include_data=True):
        umapjson = self.settings
        umapjson["type"] = "umap"
        umapjson["properties"].pop("is_template", None)
        umapjson["uri"] = request.build_absolute_uri(self.get_absolute_url())
        datalayers = []
        for datalayer in self.datalayers:
            layer = {}
            if include_data:
                with datalayer.geojson.open("rb") as f:
                    layer = json.loads(f.read())
            if datalayer.settings:
                datalayer.settings.pop("id", None)
                layer["_umap_options"] = datalayer.settings
            datalayers.append(layer)
        umapjson["layers"] = datalayers
        return umapjson

    def get_absolute_url(self):
        return reverse("map", kwargs={"slug": self.slug or "map", "map_id": self.pk})

    def get_anonymous_edit_url(self):
        signer = Signer()
        signature = signer.sign(self.pk)
        path = reverse("map_anonymous_edit_url", kwargs={"signature": signature})
        return settings.SITE_URL + path

    def get_author(self):
        return self.team or self.owner

    def is_owner(self, request=None):
        if not request:
            return False
        if request.user and self.owner == request.user:
            return True
        return self.is_anonymous_owner(request)

    def is_anonymous_owner(self, request):
        if not request or self.owner:
            # edit cookies are only valid while the map doesn't have owner
            return False
        key, value = self.signed_cookie_elements
        try:
            has_anonymous_cookie = int(request.get_signed_cookie(key, False)) == value
        except ValueError:
            has_anonymous_cookie = False
        return has_anonymous_cookie

    def can_delete(self, request=None):
        if not request:
            return False
        if self.owner and request.user != self.owner:
            return False
        if not self.owner and not self.is_anonymous_owner(request):
            return False
        return True

    def can_edit(self, request=None):
        """
        Define if a user can edit or not the instance, according to his account
        or the request.

        In owner mode:
            - only owner by default (OWNER)
            - any editor or team member if mode is COLLABORATORS
            - anyone otherwise (ANONYMOUS)
        In anonymous owner mode:
            - only owner (has ownership cookie) by default (OWNER)
            - anyone otherwise (ANONYMOUS)
        """
        can = False
        if not request:
            return False
        user = request.user
        if (
            not self.owner
            and settings.UMAP_ALLOW_ANONYMOUS
            and self.is_anonymous_owner(request)
        ):
            can = True
        elif self.edit_status == self.ANONYMOUS:
            can = True
        elif not user.is_authenticated:
            can = False
        elif user == self.owner:
            can = True
        elif self.edit_status == self.COLLABORATORS:
            if user in self.editors.all() or self.team in user.teams.all():
                can = True
        return can

    def can_view(self, request):
        if self.share_status in [Map.BLOCKED, Map.DELETED]:
            can = False
        elif self.share_status in [Map.PUBLIC, Map.OPEN]:
            can = True
        elif self.owner is None:
            can = settings.UMAP_ALLOW_ANONYMOUS and self.is_anonymous_owner(request)
        elif not request.user.is_authenticated:
            can = False
        elif request.user == self.owner:
            can = True
        else:
            restricted = self.share_status in [Map.PRIVATE, Map.DRAFT]
            can = not (
                restricted
                and request.user not in self.editors.all()
                and self.team not in request.user.teams.all()
            )
        return can

    @property
    def signed_cookie_elements(self):
        return ("anonymous_owner|%s" % self.pk, self.pk)

    def get_tilelayer(self):
        return self.tilelayer or TileLayer.get_default()

    def clone(self, **kwargs):
        new = self.__class__.objects.get(pk=self.pk)
        new.pk = None
        new.name = "%s %s" % (_("Clone of"), self.name)
        if "owner" in kwargs:
            # can be None in case of anonymous cloning
            new.owner = kwargs["owner"]
        new.save()
        for editor in self.editors.all():
            new.editors.add(editor)
        for datalayer in self.datalayers:
            datalayer.clone(map_inst=new)
        return new

    def get_tags_display(self):
        labels = dict(settings.UMAP_TAGS)
        return [(t, labels.get(t, t)) for t in self.tags]

    @classproperty
    def extra_schema(self):
        return {
            "iconUrl": {
                "default": "%sumap/img/marker.svg" % settings.STATIC_URL,
            },
            "tags": {"choices": sorted(settings.UMAP_TAGS, key=lambda i: i[0])},
        }


class Pictogram(NamedModel):
    """
    An image added to an icon of the map.
    """

    attribution = models.CharField(max_length=300)
    category = models.CharField(max_length=300, null=True, blank=True)
    pictogram = models.FileField(upload_to="pictogram")

    @property
    def json(self):
        return {
            "id": self.pk,
            "attribution": self.attribution,
            "name": self.name,
            "category": self.category,
            "src": self.pictogram.url,
        }


# Must be out of Datalayer for Django migration to run, because of python 2
# serialize limitations.
def upload_to(instance, filename):
    return instance.geojson.storage.make_filename(instance)


def set_storage():
    return storages["data"]


class DataLayer(NamedModel):
    """
    Layer to store Features in.
    """

    INHERIT = 0
    ANONYMOUS = 1
    COLLABORATORS = 2
    OWNER = 3
    DELETED = 99
    SHARE_STATUS = (
        (INHERIT, _("Inherit")),
        (DELETED, _("Deleted")),
    )
    EDIT_STATUS = (
        (INHERIT, _("Inherit")),
        (ANONYMOUS, _("Everyone")),
        (COLLABORATORS, _("Editors and team only")),
        (OWNER, _("Owner only")),
    )
    ANONYMOUS_EDIT_STATUS = (
        (INHERIT, _("Inherit")),
        (OWNER, _("Only editable with secret edit link")),
        (ANONYMOUS, _("Everyone can edit")),
    )
    uuid = models.UUIDField(unique=True, primary_key=True, editable=False)
    old_id = models.IntegerField(null=True, blank=True)
    map = models.ForeignKey(Map, on_delete=models.CASCADE)
    description = models.TextField(blank=True, null=True, verbose_name=_("description"))
    geojson = models.FileField(
        upload_to=upload_to, blank=True, null=True, storage=set_storage
    )
    display_on_load = models.BooleanField(
        default=False,
        verbose_name=_("display on load"),
        help_text=_("Display this layer on load."),
    )
    rank = models.SmallIntegerField(default=0)
    settings = models.JSONField(
        blank=True, null=True, verbose_name=_("settings"), default=dict
    )
    edit_status = models.SmallIntegerField(
        choices=EDIT_STATUS,
        default=INHERIT,
        verbose_name=_("edit status"),
    )
    share_status = models.SmallIntegerField(
        choices=SHARE_STATUS,
        default=INHERIT,
        verbose_name=_("share status"),
    )
    modified_at = models.DateTimeField(auto_now=True)

    class Meta:
        ordering = ("rank",)

    def save(self, **kwargs):
        super(DataLayer, self).save(**kwargs)
        self.geojson.storage.onDatalayerSave(self)
        if hasattr(self, "_reference_version"):
            del self._reference_version

    def delete(self, **kwargs):
        self.geojson.storage.onDatalayerDelete(self)
        return super().delete(**kwargs)

    def metadata(self, request=None):
        # Retrocompat: minimal settings for maps not saved after settings property
        # has been introduced
        metadata = self.settings
        if not metadata:
            # Fallback to file for old datalayers.
            try:
                data = json.loads(self.geojson.read().decode())
            except FileNotFoundError:
                data = {}
            metadata = data.get("_umap_options")
            if not metadata:
                metadata = {
                    "name": self.name,
                    "displayOnLoad": self.display_on_load,
                }
            # Save it to prevent file reading at each map load.
            self.settings = metadata
            # Do not update the modified_at.
            self.save(update_fields=["settings"])
        if self.old_id:
            metadata["old_id"] = self.old_id
        metadata["id"] = self.pk
        metadata["rank"] = self.rank
        metadata["permissions"] = {"edit_status": self.edit_status}
        metadata["editMode"] = "advanced" if self.can_edit(request) else "disabled"
        metadata["_referenceVersion"] = self.reference_version
        return metadata

    def clone(self, map_inst=None):
        new = self.__class__.objects.get(pk=self.pk)
        new._state.adding = True
        new.pk = uuid.uuid4()
        if map_inst:
            new.map = map_inst
        new.geojson = File(new.geojson.file.file, name="tmpname")
        new.save()
        return new

    @property
    def reference_version(self):
        if not hasattr(self, "_reference_version"):
            self._reference_version = self.geojson.storage.get_reference_version(self)
        return self._reference_version

    @property
    def versions(self):
        return self.geojson.storage.list_versions(self)

    def get_version(self, ref):
        return self.geojson.storage.get_version(ref, self)

    def get_version_path(self, ref):
        return self.geojson.storage.get_version_path(ref, self)

    def can_edit(self, request=None):
        """
        Define if a user can edit or not the instance, according to his account
        or the request.
        """
        if self.edit_status == self.INHERIT:
            return self.map.can_edit(request)
        can = False
        if not request:
            return False
        user = request.user
        if not self.map.owner:
            if settings.UMAP_ALLOW_ANONYMOUS and self.map.is_anonymous_owner(request):
                can = True
        if self.edit_status == self.ANONYMOUS:
            can = True
        elif user.is_authenticated and user == self.map.owner:
            can = True
        elif user.is_authenticated and self.edit_status == self.COLLABORATORS:
            if user in self.map.editors.all() or self.map.team in user.teams.all():
                can = True
        return can

    def move_to_trash(self):
        self.share_status = DataLayer.DELETED
        self.save()


class Star(models.Model):
    at = models.DateTimeField(auto_now=True)
    map = models.ForeignKey(Map, on_delete=models.CASCADE)
    by = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name="stars", on_delete=models.CASCADE
    )
